(function () {
    'use strict';

    // file: storage_service.js
    function getItem(storage, key, defaultValue) {
        return parseValue(storage.getItem(key), defaultValue);
    }

    function getItems(storage, keyRe) {
        return Object.keys(storage).filter(function (key) {
            return keyRe.test(key);
        }).map(function (key) {
            return { key: key, value: getItem(storage, key) };
        });
    }

    function setItem(storage, key, value) {
        storage.setItem(key, serializeValue(value));
    }

    function removeItem(storage, key) {
        storage.removeItem(key);
    }

    function parseValue(valueStr, defaultValue) {
        return valueStr ? JSON.parse(valueStr) : defaultValue;
    }

    function serializeValue(value) {
        return JSON.stringify(value);
    }

    // file: custom_events.js
    function lsChange(key, oldValue, newValue) {
        var isFromForeignWindow = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : false;

        document.dispatchEvent(new CustomEvent('lsChange', {
            detail: { key: key, oldValue: oldValue, newValue: newValue, isFromForeignWindow: isFromForeignWindow }
        }));
    }

    // file: local_storage_service.js

    // because of this bug https://bugzilla.mozilla.org/show_bug.cgi?id=1453699
    // which affects Tor Browser users
    // we need a temporary workaround: we ask other open tabs to send their localStorage to this tab
    // if we don't receive anything during 20ms we assume that there're no other open tabs.
    // we use fake localStorage object and synchronize it across tabs using BroadcastChannel.
    // When this bug is fixed in Tor Browser, this code will be removed.
    var isLsBugWorkaroundNeeded = navigator.userAgent === "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:68.0) Gecko/20100101 Firefox/68.0" || navigator.userAgent === "Mozilla/5.0 (Windows NT 10.0; rv:68.0) Gecko/20100101 Firefox/68.0" || navigator.userAgent === "Mozilla/5.0 (Windows NT 6.1; rv:60.0) Gecko/20100101 Firefox/60.0" || navigator.userAgent === "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:78.0) Gecko/20100101 Firefox/78.0";
    var wa = {}; // workaround

    function initLocalStorageService(callback) {
        var onInitialized = function onInitialized() {
            addStorageEventListener();
            callback();
        };

        if (isLsBugWorkaroundNeeded) {
            wa.fakeLocalStorage = {};
            Object.keys(localStorage).forEach(function (key) {
                return wa.fakeLocalStorage[key] = localStorage[key];
            });

            wa.bc = new BroadcastChannel('ls');
            wa.bc.postMessage({ type: 'send your ls' });

            var isLsReceived = false;
            wa.bc.addEventListener('message', function (e) {
                if (e.data.type === 'send your ls') {
                    wa.bc.postMessage({ type: 'ls', localStorage: wa.fakeLocalStorage });
                }

                if (e.data.type === 'ls') {
                    if (isLsReceived) return;

                    isLsReceived = true;
                    var receivedLs = e.data.localStorage;
                    Object.keys(receivedLs).forEach(function (key) {
                        if (receivedLs[key] !== undefined) {
                            wa.fakeLocalStorage[key] = receivedLs[key];
                            localStorage[key] = receivedLs[key];
                        }
                    });
                    onInitialized();
                }

                // storage event emulation
                if (e.data.type === 'storage') {
                    if (e.data.newValue === undefined) {
                        delete wa.fakeLocalStorage[e.data.key];
                        delete localStorage[e.data.key];
                    } else {
                        wa.fakeLocalStorage[e.data.key] = e.data.newValue;
                        localStorage[e.data.key] = e.data.newValue;
                    }
                    onStorageHandler(e.data);
                }
            });

            setTimeout(function () {
                if (!isLsReceived) {
                    console.log('timeout');
                    isLsReceived = true;
                    onInitialized();
                }
            }, 20);
        } else {
            onInitialized();
        }
    }

    function getItem$1(key, defaultValue) {
        if (isLsBugWorkaroundNeeded) {
            return parseValue(wa.fakeLocalStorage[key], defaultValue);
        }

        return getItem(localStorage, key, defaultValue);
    }

    function getItems$1(keyRe) {
        if (isLsBugWorkaroundNeeded) {
            return Object.keys(wa.fakeLocalStorage).filter(function (key) {
                return keyRe.test(key);
            }).map(function (key) {
                return { key: key, value: getItem$1(key) };
            });
        }

        return getItems(localStorage, keyRe);
    }

    function setItem$1(key, value, defaultValue) {
        var oldValue = getItem$1(key, defaultValue);

        if (isLsBugWorkaroundNeeded) {
            var oldValueStr = serializeValue(oldValue);
            var newValueStr = serializeValue(value);
            wa.fakeLocalStorage[key] = newValueStr;

            if (oldValueStr !== newValueStr) {
                // console.log('send newVal', newValueStr);
                wa.bc.postMessage({
                    type: 'storage', key: key,
                    oldValue: oldValueStr,
                    newValue: newValueStr
                });
            }
        }

        setItem(localStorage, key, value);

        // todo: this won't work for objects
        if (oldValue !== value) lsChange(key, oldValue, value);
    }

    function removeItem$1(key) {
        if (isLsBugWorkaroundNeeded) {
            var oldValueStr = wa.fakeLocalStorage[key];
            delete wa.fakeLocalStorage[oldValueStr];

            wa.bc.postMessage({
                type: 'storage', key: key,
                oldValue: oldValueStr
            });
        }

        removeItem(localStorage, key);
    }

    function addStorageEventListener() {
        if (isLsBugWorkaroundNeeded) return;

        window.addEventListener('storage', onStorageHandler, false);
    }

    function onStorageHandler(e) {
        // console.log('received newVal', e.newValue);
        // console.log('onStorage', e.key, e.oldValue, e.newValue); // todo: delete
        var oldValue = parseValue(e.oldValue, null);
        var newValue = parseValue(e.newValue, null);

        lsChange(e.key, oldValue, newValue, true);
    }

    // file: config.js
    // we store here parameters that cannot be changed by a user
    var config = {
        version: "0.3.1",
        maxPostLength: 9000,
        maxSubjectLength: 128,
        maxBoardNameLength: 3,
        boardNamePattern: '[a-zA-Z]{1,3}',
        basePath: '/ui2',
        showIgnoreCyrillicShitFilter: false,
        pageSize: {
            general: 100,
            warp: 20
        },
        formatting: {
            maxNestingLevel: 10
        },
        katex: {
            cssPath: "/static/katex/katex.min.css",
            scriptPath: "/static/katex/katex.min.js"
        },
        hljs: {
            cssPath: "/static/hljs/styles/vs.css",
            scriptPath: "/static/hljs/highlight.pack.js"
        },
        pagination: {
            siblingCount: 3
        },
        sse: {
            type: {
                post: "post",
                connected: "connected",
                boardDescription: "board-description"
            },
            continueSubscriptionTimeout: 30000, // ms
            reconnectTimeout: 1000, // ms
            broadcastChannel: 'sse'
        },
        notification: {
            type: {
                success: 'success',
                error: 'error',
                warning: 'warning',
                info: 'info'
            },
            autoHideAfter: 2000 // ms
        },
        chat: {
            boardName: 'cht',
            threadId: 'a237b4ad-bdbd-4fa2-9abf-1f646fdcf40a',
            keepScrollBottomThreshold: 50 // px
        },
        defaultSettings: {
            showOnlyChatPostsInChat: false,
            enableEndlessScroll: true,
            ignoreCyrillicShit: false,
            isKatexEnabled: false,
            highlightCode: false,
            showImages: true,
            embedBitchute: false,
            maxImageHeight: 200,
            maxImageContentLength: 2000000,
            maxEntropy: 6,
            regexpFilters: '://imagetwist.com\n://cuts-url.com\n',
            colorsFilters: ''
        },
        tabHeartbeatTimeout: 500, // ms
        submissionTimeout: 20000 // ms
    };

    window.$ = function (selector) {
        return document.querySelector(selector);
    };
    window.$a = function (selector) {
        return document.querySelectorAll(selector);
    };
    Element.prototype.$ = function (selector) {
        return this.querySelector(selector);
    };
    Element.prototype.$a = function (selector) {
        return this.querySelectorAll(selector);
    };

    HTMLCollection.prototype.forEach = function (fn) {
        return Array.prototype.forEach.call(this, fn);
    };

    HTMLCollection.prototype.indexOf = function (element) {
        return Array.prototype.indexOf.call(this, element);
    };

    Element.prototype.getClosestByClassName = function (className) {
        for (var parent = this; parent; parent = parent.parentElement) {
            if (parent.classList.contains(className)) {
                return parent;
            }
        }
        return null;
    };

    Element.prototype.insertBrBefore = function (element) {
        var br = document.createElement("br");
        this.insertBefore(br, element);
        return br;
    };

    Element.prototype.insertTextBefore = function (text, element) {
        this.insertBefore(document.createTextNode(text), element);
    };

    Element.prototype.appendText = function (x) {
        var text = x.toString();
        if (text) this.appendChild(document.createTextNode(text));
    };

    Element.prototype.appendAny = function (x) {
        var _this = this;

        if (x instanceof Element) {
            this.appendChild(x);
        } else if (typeof x === 'string') {
            this.appendText(x);
        } else if (Array.isArray(x)) {
            x.forEach(function (item) {
                return _this.appendAny(item);
            });
        } else {
            console.error(this, x);
            throw Error("appendAny: invalid argument");
        }
        return this;
    };

    Element.prototype.appendBefore = function (x) {
        x.parentNode.insertBefore(this, x);
    };

    Element.prototype.appendAfter = function (x) {
        x.parentNode.insertBefore(this, x.nextSibling);
    };

    DocumentFragment.prototype.appendAfter = function (x) {
        x.parentNode.insertBefore(this, x.nextSibling);
    };

    Element.prototype.scrollToBottom = function () {
        this.scrollTop = this.scrollHeight;
    };

    Element.prototype.hide = function () {
        this.classList.add('hidden');
    };

    Element.prototype.show = function () {
        this.classList.remove('hidden');
    };

    Element.prototype.isHidden = function () {
        return this.classList.contains('hidden');
    };

    Array.prototype.last = function () {
        return this[this.length - 1];
    };

    Number.prototype.getNthByte = function (n) {
        var shift = (n - 1) * 8;
        return (this >> shift) % 0x100;
    };

    Number.prototype.toRGB = function () {
        return "rgb(" + this.getNthByte(3) + ", " + this.getNthByte(2) + ", " + this.getNthByte(1) + ")";
    };

    Number.prototype.toHSL = function () {
        var h = (this >> 16) % 360;
        var s = this.getNthByte(2) % 101;
        var l = this.getNthByte(1) % 101;
        return "hsl(" + h + ", " + s + "%, " + l + "%)";
    };

    Promise.prototype.thenA = function (onFulfilled, onRejected) {
        return this.then(function (f) {
            return f(onFulfilled);
        }, onRejected);
    };

    Date.prototype.withoutTime = function () {
        var d = new Date(this);
        d.setHours(0, 0, 0, 0);
        return d;
    };

    function promiseTimeout(promise, timeout, timeoutCallback) {
        var timeoutId = setTimeout(timeoutCallback, timeout);

        return promise.then(function (result) {
            clearTimeout(timeoutId); // prevent calling timeoutCallback
            return result;
        }).catch(function (e) {
            clearTimeout(timeoutId);
            throw e;
        });
    }

    function tryDecodeURI(encodedURI) {
        try {
            return decodeURI(encodedURI);
        } catch (e) {
            return encodedURI;
        }
    }

    function importScript(src) {
        return new Promise(function (resolve, reject) {
            var scriptE = document.createElement("script");
            scriptE.src = src;
            scriptE.async = true;
            scriptE.onload = resolve;
            scriptE.onerror = reject;
            document.head.appendChild(scriptE);
        });
    }

    function importCss(src) {
        return new Promise(function (resolve, reject) {
            var linkE = document.createElement("link");
            linkE.rel = "stylesheet";
            linkE.href = src;
            linkE.onload = resolve;
            linkE.onerror = reject;
            document.head.appendChild(linkE);
        });
    }

    function formatDate(date) {
        return date.toISOString().slice(0, 19).replace('T', ' ');
    }

    function formatTime(date) {
        return formatDate(date).split(' ')[1];
    }

    var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

    function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var ServiceLoader = function () {
        function ServiceLoader(cssUrl, scriptUrl) {
            _classCallCheck(this, ServiceLoader);

            this._cssUrl = cssUrl;
            this._scriptUrl = scriptUrl;

            this._isServiceLoaded = false;
            this._isServiceEnabled = false;
            this._serviceLoadPromise = null;
        }

        _createClass(ServiceLoader, [{
            key: "enableService",
            value: function enableService(isEnabled) {
                var _this = this;

                this._isServiceEnabled = isEnabled;

                // load it if enabled
                if (!isEnabled) {
                    return Promise.resolve();
                }

                if (this._isServiceLoaded) {
                    // already loaded
                    return Promise.resolve();
                }

                if (this._serviceLoadPromise) {
                    // it is loading at the moment
                    return this._serviceLoadPromise;
                }

                this._serviceLoadPromise = this._loadService().then(function () {
                    return _this._isServiceLoaded = true;
                });
                return this._serviceLoadPromise;
            }
        }, {
            key: "_loadService",
            value: function _loadService() {
                return Promise.all([importCss(this._cssUrl), importScript(this._scriptUrl)]);
            }
        }, {
            key: "isServiceEnabled",
            get: function get() {
                return this._isServiceEnabled;
            }
        }]);

        return ServiceLoader;
    }();

    // file: katex_service.js

    var serviceLoader = new ServiceLoader(config.katex.cssPath, config.katex.scriptPath);

    function enableKatex(isEnabled) {
        return serviceLoader.enableService(isEnabled);
    }

    function renderKatex(formula, element) {
        var plain = "$" + formula + "$";

        if (!serviceLoader.isServiceEnabled) {
            element.appendText(plain);
            return;
        }

        try {
            katex.render(formula, element, { throwOnError: false });
        } catch (e) {
            // console.warn('renderKatex exception:', e);
            element.appendText(plain);
        }
    }

    // file: filter_service.js
    function containCyrillic(str) {
        for (var i = 0; i < str.length; i++) {
            var charCode = str.charCodeAt(i);
            if (charCode > 1023 && charCode < 1280) return true;
        }
        return false;
    }

    function getShannonEntropy(text) {
        var set = {};

        text.split('').forEach(function (c) {
            return set[c] ? set[c]++ : set[c] = 1;
        });

        return Object.keys(set).reduce(function (acc, c) {
            var p = set[c] / text.length;
            return acc - p * (Math.log(p) / Math.log(2));
        }, 0);
    }

    var regexpFiltersStr = '';
    var filtersRegexps = []; // todo: use Set
    function parseRegexpFilters(str) {
        // we memorize the last result because we don't want to parse the same regexps for each post
        if (str === regexpFiltersStr) return filtersRegexps;

        filtersRegexps = [];
        regexpFiltersStr = str || '';

        regexpFiltersStr.split('\n').filter(function (x) {
            return x;
        }).forEach(function (x) {
            try {
                filtersRegexps.push(new RegExp(x, 'i'));
            } catch (e) {
                console.warn("Invalid regexp: ", x, e);
            }
        });

        return filtersRegexps;
    }

    var colorsFiltersStr = '';
    var colorsFilters = [];
    function parseColorsFilters(str) {
        if (str === colorsFiltersStr) return colorsFilters;

        colorsFilters = [];
        colorsFiltersStr = str || '';

        var colorsFiltersSet = new Set();

        colorsFiltersStr.split('\n').filter(function (x) {
            return x;
        }).forEach(function (x) {
            return colorsFiltersSet.add(x);
        });

        colorsFilters = Array.from(colorsFiltersSet).map(function (x) {
            var match = /^([0-9]+):([0-9]+)$/.exec(x);
            if (!match) return;

            return {
                bg: Number.parseInt(match[1]),
                fg: Number.parseInt(match[2])
            };
        }).filter(function (x) {
            return x;
        });

        return colorsFilters;
    }

    // file: settings_service.js

    function getMaxContentLength() {
        return getItem$1("MaxContentLength", config.defaultSettings.maxImageContentLength);
    }

    function getMaxImgHeight() {
        return getItem$1("MaxImgHeight", config.defaultSettings.maxImageHeight);
    }

    function doShowImgs() {
        return getItem$1("ShowImgsP", config.defaultSettings.showImages);
    }

    function doEmbedBitchute() {
        return getItem$1("EmbedBitchuteP", config.defaultSettings.embedBitchute);
    }

    function doIgnoreCyrillic() {
        return getItem$1("IgnoreCyrillicP", config.defaultSettings.ignoreCyrillicShit);
    }

    function doEnableKatex() {
        return getItem$1("enableKatexP", config.defaultSettings.isKatexEnabled);
    }

    function doHighlightCode() {
        return getItem$1("highlightCodeP", config.defaultSettings.highlightCode);
    }

    function getMaxEntropy() {
        return getItem$1("maxEntropyP", config.defaultSettings.maxEntropy);
    }

    function getRegexpFilters() {
        // todo: rename filtersP to regexpFilters, but be aware that some clients already use filtersP!
        return getItem$1("filtersP", config.defaultSettings.regexpFilters);
    }

    function getColorsFilters() {
        return getItem$1("colorsFilters", config.defaultSettings.colorsFilters);
    }

    // chat specific
    function doShowOnlyChatPosts() {
        return getItem$1("showOnlyChatPostsP", config.defaultSettings.showOnlyChatPostsInChat);
    }

    function doEnableEndlessScroll() {
        return getItem$1("enableEndlessScrollP", config.defaultSettings.enableEndlessScroll);
    }

    function setMaxContentLength(value) {
        setItem$1("MaxContentLength", value);
    }

    function setMaxImgHeight(value) {
        setItem$1("MaxImgHeight", value);
    }

    function setShowImgsFlag(value) {
        setItem$1("ShowImgsP", value);
    }

    function setEmbedBitchuteFlag(value) {
        setItem$1("EmbedBitchuteP", value);
    }

    function setIgnoreCyrillicFlag(value) {
        setItem$1("IgnoreCyrillicP", value);
    }

    function setEnableKatexFlag(value) {
        return setItem$1("enableKatexP", value);
    }

    function setHighlightCodeFlag(value) {
        return setItem$1("highlightCodeP", value);
    }

    function setMaxEntropy(value) {
        return setItem$1("maxEntropyP", value || 0);
    }

    function setRegexpFilters(value) {
        return setItem$1("filtersP", value || "");
    }

    function setColorsFilters(value) {
        return setItem$1("colorsFilters", value || "");
    }

    // chat specific
    function setShowOnlyChatPostsFlag(value) {
        setItem$1("showOnlyChatPostsP", value);
    }

    function setEnableEndlessScrollFlag(value) {
        setItem$1("enableEndlessScrollP", value);
    }

    function getSubmissionTimeout() {
        return config.submissionTimeout;
    }

    function getSubmissionTimeoutMsg() {
        return "It takes too long to send the message. " + "Check your connection.";
    }

    function getSubmissionFailureMsg() {
        return "Message failed to send. " + "Check your connection.";
    }

    function isPostIgnored(post) {
        if (isIgnored(post.text)) return true;

        return parseColorsFilters(getColorsFilters()).some(function (_ref) {
            var bg = _ref.bg,
                fg = _ref.fg;
            return post.bg === bg && post.fg === fg;
        });
    }

    function isThreadIgnored(thread, op) {
        return isPostIgnored(op) || isIgnored(thread.subject);
    }

    function isIgnored(text) {
        if (doIgnoreCyrillic() && containCyrillic(text)) {
            return true;
        }

        var maxEntropy = getMaxEntropy();
        var entropy = void 0;
        if (maxEntropy > 0 && (entropy = getShannonEntropy(text)) > maxEntropy) {
            console.debug("Ignored. Entropy:", entropy, text.substring(0, 500));
            return true;
        }

        return parseRegexpFilters(getRegexpFilters()).some(function (re) {
            return re.test(text);
        });
    }

    // file: uuid_service.js
    function toSimpleUuid(x) {
        return x.replace(/-/g, "").toLowerCase();
    }

    function toShortcutUuid(uuid) {
        return uuid.substring(0, 5).toLowerCase();
    }

    function toHyphenatedUuid(uuid) {
        uuid = toSimpleUuid(uuid);
        return [uuid.substring(0, 8), uuid.substring(8, 12), uuid.substring(12, 16), uuid.substring(16, 20), uuid.substring(20, 32)].join("-");
    }

    var hyphenatedUuidPattern = '[\\da-f]{8}-[\\da-f]{4}-4[\\da-f]{3}-[89ab][\\da-f]{3}-[\\da-f]{12}';
    var simpleUuidPattern = '[\\da-f]{8}[\\da-f]{4}4[\\da-f]{3}[89ab][\\da-f]{3}[\\da-f]{12}';
    var uuidPattern = "(" + simpleUuidPattern + "|" + hyphenatedUuidPattern + ")";

    // file: text_parsing_service.js

    function parsePostReference(text) {
        if (!text.startsWith('@')) return null;

        var uuid = parseUuid(text.slice(1));
        if (!uuid) return null;
        uuid.match = toHyphenatedUuid(uuid.match);
        return uuid;
    }

    function parseUuid(text) {
        var simpleUuid = parseSimpleUuid(text);
        if (simpleUuid) return simpleUuid;

        var hyphenatedUuid = parseHyphenatedUuid(text);
        if (hyphenatedUuid) return hyphenatedUuid;

        return null;
    }

    function parseSimpleUuid(text) {
        var simpleUuidRe = /^[\da-f]{8}[\da-f]{4}4[\da-f]{3}[89ab][\da-f]{3}[\da-f]{12}/i;
        return parseByRe(simpleUuidRe, text);
    }

    function parseHyphenatedUuid(text) {
        var hyphenatedUuidRe = /^[\da-f]{8}-[\da-f]{4}-4[\da-f]{3}-[89ab][\da-f]{3}-[\da-f]{12}/i;
        return parseByRe(hyphenatedUuidRe, text);
    }

    function parseByRe(re, text) {
        var match = re.exec(text);
        if (!match) return null;
        return { match: match[0], rest: text.slice(match[0].length) };
    }

    // file: location_service.js

    function getPathName() {
        return location.pathname.substring(config.basePath.length);
    }

    function getPathParts() {
        return getPathName().split('/');
    }

    function getSelectedBoardName() {
        var name = getPathParts()[1].toLowerCase();
        return (/^[a-z]{1,3}$/.test(name) ? name : null
        );
    }

    function getSelectedThreadId() {
        if (!getSelectedBoardName()) return null;

        return getThreadId();
    }

    function getThreadId() {
        var threadId = parseUuid(getPathParts()[2]);
        return threadId ? threadId.match : null;
    }

    function getPage() {
        var pageNumber = parseInt(getQueryStrings().p);
        return isNaN(pageNumber) ? null : pageNumber;
    }

    function getPostId() {
        var match = /(@.+)/.exec(getPathName());
        if (!match) return null;
        return parsePostReference(match[1]).match;
    }

    function activateTarget() {
        if (location.hash) {
            // noinspection SillyAssignmentJS
            location.hash = location.hash;
        }
    }

    function getQueryStrings() {
        var result = {};
        location.search.substring(1).split('&').forEach(function (x) {
            var idx = x.indexOf('=');
            if (idx > 0) {
                result[decodeURIComponent(x.substring(0, idx))] = decodeURIComponent(x.substring(idx + 1));
            } else {
                result[decodeURIComponent(x)] = null;
            }
        });
        return result;
    }

    function _classCallCheck$1(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    // file: models.js
    var Board = function Board(name, description) {
        _classCallCheck$1(this, Board);

        this.name = name;
        this.description = description;
    };

    var Thread = function Thread(threadId, subject, postCount, board) {
        _classCallCheck$1(this, Thread);

        this.threadId = threadId;
        this.subject = subject;
        this.postCount = postCount;
        this.board = board;
    };

    var Post = function Post(postId, postDate, bg, fg, text, hiddenByOp, user, thread, index) {
        _classCallCheck$1(this, Post);

        this.postId = postId;
        this.postDate = postDate;
        this.bg = bg;
        this.fg = fg;
        this.text = text;
        this.hiddenByOp = hiddenByOp;
        this.user = user;
        this.thread = thread;
        this.index = index; // the index of the post in the thread
        this.ephemeral = false;
    };

    var ThreadSlice = function ThreadSlice(thread, op, posts) {
        _classCallCheck$1(this, ThreadSlice);

        this.thread = thread;
        this.op = op;
        this.posts = posts;
    };

    var WarpSlice = function WarpSlice(board, shellThreads) {
        _classCallCheck$1(this, WarpSlice);

        this.board = board;
        this.shellThreads = shellThreads; // thread slices with less than 3 posts
    };

    var ShortUserInfo = function ShortUserInfo(nickname, banned) {
        _classCallCheck$1(this, ShortUserInfo);

        this.nickname = nickname;
        this.banned = banned;
    };

    var NewPost = function NewPost(clientId, text, threadId, subject, boardName) {
        _classCallCheck$1(this, NewPost);

        this.clientId = clientId;
        this.text = text;
        this.threadId = threadId;
        this.subject = subject;
        this.boardName = boardName;
    };

    var UpdatedHiddenByOp = function UpdatedHiddenByOp(clientId, postId, hiddenByOp) {
        _classCallCheck$1(this, UpdatedHiddenByOp);

        this.clientId = clientId;
        this.postId = postId;
        this.hiddenByOp = hiddenByOp;
    };

    var SuccessResponse = function SuccessResponse(data, skipped, taken, total) {
        _classCallCheck$1(this, SuccessResponse);

        this.data = data;
        this.skipped = skipped;
        this.taken = taken;
        this.total = total;
    };

    var RequestBody = function RequestBody(data) {
        _classCallCheck$1(this, RequestBody);

        this.data = data;
    };

    var ErrorResponse = function ErrorResponse(errorCode, message) {
        _classCallCheck$1(this, ErrorResponse);

        this.errorCode = errorCode;
        this.message = message;
    };

    // file: client_service.js

    var clientIdLsKey = 'betaClientId';

    function getOrCreateClientId() {
        var clientId = getItem$1(clientIdLsKey, null);

        if (clientId) return Promise.resolve(clientId);

        clientId = newClientId();
        setItem$1(clientIdLsKey, clientId);

        return Promise.resolve(clientId);
    }

    function updateClientId() {
        return setClientId(newClientId());
    }

    function setClientId(clientId) {
        setItem$1(clientIdLsKey, clientId);

        return Promise.resolve(clientId);
    }

    function newClientId() {
        return Math.random().toString(36).substring(2);
    }

    // file: parsers.js

    function parseBoard(obj) {
        return new Board(obj.name, obj.description);
    }

    function parseThread(obj) {
        return new Thread(obj.threadId, obj.subject, obj.postCount, obj.board ? parseBoard(obj.board) : null);
    }

    function parsePost(obj) {
        return new Post(obj.postId, new Date(obj.postDate), obj.bg, obj.fg, obj.text, obj.hiddenByOp, parseShortUserInfo(obj.user), obj.thread ? parseThread(obj.thread) : null, obj.index);
    }

    function parseThreadSlice(obj) {
        return new ThreadSlice(parseThread(obj.thread), parsePost(obj.op), obj.posts.map(parsePost));
    }

    function parseWarpSlice(obj) {
        return new WarpSlice(obj.board ? parseBoard(obj.board) : null, obj.shellThreads.map(parseThreadSlice));
    }

    function parseShortUserInfo(obj) {
        return new ShortUserInfo(obj.nickname, obj.banned);
    }

    function parseSuccessResponse(obj) {
        return function (dataParser) {
            return new SuccessResponse(dataParser(obj.data), obj.skipped, obj.taken, obj.total);
        };
    }

    function parseErrorResponse(obj) {
        return new ErrorResponse(obj.errorCode, obj.message);
    }

    // file: url_service.js

    var prefix = config.basePath;
    var api = "s_api";

    function getRecentPostsApiUrl() {
        return "/" + api + "/posts?" + toQuery({
            includeThread: true,
            includeBoard: true,
            sortByDate: "desc"
        });
    }

    function getPostsBeforeApiUrl(postId) {
        return "/" + api + "/posts?" + toQuery({
            includeThread: true,
            includeBoard: true,
            sortByDate: "desc",
            beforePost: postId
        });
    }

    function getPostsAfterApiUrl(postId) {
        return "/" + api + "/posts?" + toQuery({
            includeThread: true,
            includeBoard: true,
            sortByDate: "asc",
            afterPost: postId
        });
    }

    function getLogsPageApiUrl(pageNumber) {
        return "/" + api + "/posts?" + toQuery({
            includeThread: true,
            includeBoard: true,
            includeTotal: true,
            sortByDate: "desc",
            skip: pageNumber * config.pageSize.general
        });
    }

    function getPostApiUrl(postId, includeIndex) {
        return "/" + api + "/posts/" + postId + "?" + toQuery({
            includeThread: true,
            includeBoard: true,
            includeIndex: includeIndex
        });
    }

    function getBoardUrl(boardName) {
        return prefix + "/" + boardName + "/";
    }

    function getThreadUrl(boardName, threadId) {
        var pageNumber = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : null;
        var postId = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : null;

        var path = prefix + "/" + boardName + "/" + threadId + "/";

        var pathAndQuery = pageNumber === null ? path : path + "?" + toQuery({ p: pageNumber });

        return postId === null ? pathAndQuery : pathAndQuery + "#p" + postId;
    }

    function getWarpUrl(boardName, pageNumber) {
        return prefix + ((boardName ? "/" + boardName + "/" : "/chan") + (pageNumber ? "?" + toQuery({ p: pageNumber }) : ""));
    }

    function getLogsUrl(pageNumber) {
        return prefix + "/logs?" + toQuery({ p: pageNumber });
    }

    function getPostLinkUrl(postId) {
        return prefix + "/@" + postId;
    }

    function getThreadPageApiUrl(threadId, pageNumber) {
        // null means the last page
        // + 1 because we want to skip OP

        return "/" + api + "/threadSlices/" + threadId + "?" + toQuery({
            skip: (pageNumber === null ? 0 : pageNumber * config.pageSize.general) + 1,
            lastSlice: pageNumber === null,
            includeBoard: true
        });
    }

    function getWarpPageApiUrl(boardName, pageNumber) {
        return "/" + api + "/warpSlices?" + toQuery({
            board: boardName,
            skip: pageNumber * config.pageSize.warp,
            includeBoard: !boardName
        });
    }

    function getPublishApiUrl() {
        return "/" + api + "/posts";
    }

    function getUpdateHiddenByOpApiUrl() {
        return "/" + api + "/posts";
    }

    function getUpdateBoardDescriptionApiUrl(boardName) {
        return "/" + api + "/boards";
    }

    function getSubscribeApiUrl() {
        return "/" + api + "/subscribe";
    }

    function getContinueSubscriptionApiUrl(subscriptionId) {
        return "/" + api + "/continueSubscription/" + subscriptionId;
    }

    function toQuery(obj) {
        return Object.keys(obj).filter(function (k) {
            return obj[k] !== null && obj[k] !== undefined;
        }).map(function (k) {
            return encodeURIComponent(k) + "=" + encodeURIComponent(obj[k]);
        }).join('&');
    }

    // file: cookie_service.js

    function getCookie(name) {
        return document.cookie.split(";").map(function (kv) {
            return kv.trim().split('=');
        }).filter(function (kv) {
            return kv[0] === name;
        }).map(function (kv) {
            return kv[1];
        })[0];
    }

    // file: access_token_service.js

    var cookieKey = "accessToken";

    function getAccessToken() {
        return getCookie(cookieKey);
    }

    var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

    function getRecentPosts() {
        return getJson(getRecentPostsApiUrl()).thenA(function (ps) {
            return ps.map(parsePost).reverse();
        });
    }

    function getPostsBefore(postId) {
        return getJson(getPostsBeforeApiUrl(postId)).thenA(function (ps) {
            return ps.map(parsePost).reverse();
        });
    }

    function getPostsAfter(postId) {
        return getJson(getPostsAfterApiUrl(postId)).thenA(function (ps) {
            return ps.map(parsePost);
        });
    }

    function getLogsPage(pageNumber) {
        return getJson(getLogsPageApiUrl(pageNumber)).thenA(function (ps) {
            return ps.map(parsePost);
        });
    }

    function getPost(postId, includeIndex) {
        return getJson(getPostApiUrl(postId, includeIndex)).thenA(parsePost);
    }

    function getThreadPage(threadId, pageNumber) {
        // null means the last page

        return getJson(getThreadPageApiUrl(threadId, pageNumber)).thenA(parseThreadSlice);
    }

    function getWarpPage(boardName, pageNumber) {
        return getJson(getWarpPageApiUrl(boardName, pageNumber)).thenA(parseWarpSlice);
    }

    function updateBoardDescription(board) {
        return patchJson(getUpdateBoardDescriptionApiUrl(), new RequestBody(board)).thenA(function (message) {
            return message;
        });
    }

    function updateHiddenByOp(updatedHiddenByOp) {
        return patchJson(getUpdateHiddenByOpApiUrl(), new RequestBody(updatedHiddenByOp)).thenA(function (message) {
            return message;
        });
    }

    function publish(newPost) {
        return postJson(getPublishApiUrl(), new RequestBody(newPost)).thenA(function (p) {
            return parsePost(p);
        });
    }

    function continueSubscription(subscriptionId) {
        return postJson(getContinueSubscriptionApiUrl(subscriptionId)).thenA(function (message) {
            return message;
        });
    }

    function getJson(url) {
        return handleFetchPromise(fetch(url));
    }

    function postJson(url, data) {
        return httpRequestJson(url, 'POST', data);
    }

    function patchJson(url, data) {
        return httpRequestJson(url, 'PATCH', data);
    }

    function httpRequestJson(url, method, data) {
        var headers = {
            'Accept': 'application/json',
            'Content-Type': 'application/json'
        };
        if (['POST', 'PATCH', 'DELETE'].includes(method)) {
            var accessToken = getAccessToken();
            if (accessToken) headers['Authorization'] = accessToken;
        }
        return handleFetchPromise(fetch(url, {
            method: method,
            headers: headers,
            body: JSON.stringify(data)
        }));
    }

    function handleFetchPromise(fetchPromise) {
        return fetchPromise.then(function (response) {
            return Promise.all([Promise.resolve(response), response.json()]);
        }).then(function (_ref) {
            var _ref2 = _slicedToArray(_ref, 2),
                response = _ref2[0],
                json = _ref2[1];

            if (response.ok) return json;

            throw parseErrorResponse(json);
        }).then(function (json) {
            return parseSuccessResponse(json);
        });
    }

    // file: post_cache_service.js

    var cachedPosts = {};

    function get(postId) {
        // console.log(cachedPosts);
        return cachedPosts[toSimpleUuid(postId)];
    }

    function saveForThreadSlice(threadSlice) {
        save(threadSlice.op);
        saveMany(threadSlice.posts);
        save(threadSlice.prevPost);
        return threadSlice;
    }

    function saveForWarpSlice(warpSlice) {
        warpSlice.shellThreads.forEach(saveForThreadSlice);
        return warpSlice;
    }

    function save(post) {
        if (!post) return post;

        // do not cache if thread or board not available
        if (!post.thread || !post.thread.board) return post;

        cachedPosts[toSimpleUuid(post.postId)] = post;
        return post;
    }

    function saveMany(posts) {
        posts.forEach(save);
        return posts;
    }

    // file: alive_tabs_service.js

    var tabCreatedAt = new Date().getTime();
    var tabId = "tab_" + tabCreatedAt;
    var isEventSourceTab = false;
    var isEventSourceListeners = [];

    function heartbeat() {
        var aliveTabs = getOtherAliveTabs();
        var oldIsEventSourceTab = isEventSourceTab;

        // this tab will be used to connect to SSE if
        // there's no other connected tab at the moment and it's the oldest tab
        isEventSourceTab = !aliveTabs.some(function (x) {
            return x.value.isEventSource;
        }) && aliveTabs.every(function (x) {
            return x.value.createdAt > tabCreatedAt;
        });

        setItem$1(tabId, {
            createdAt: tabCreatedAt,
            aliveAt: new Date().getTime(),
            isEventSource: isEventSourceTab
        });

        if (isEventSourceTab !== oldIsEventSourceTab) {
            isEventSourceListeners.forEach(function (listener) {
                return listener(isEventSourceTab);
            });
        }
    }

    function initAliveTabsService() {
        cleanOldTabs();
        heartbeat();
        var intervalId = setInterval(function () {
            return heartbeat();
        }, config.tabHeartbeatTimeout);

        addEventListener('beforeunload', function (e) {
            clearInterval(intervalId);
            removeItem$1(tabId);
        });
    }

    function getTabs() {
        var tabs = getItems$1(/^tab_/);
        // const now = new Date().getTime();
        // console.log(tabs.map(t => {
        //     return {tab: t.key, at: t.value.aliveAt, dur: (now - t.value.aliveAt)};
        // }));
        return tabs;
    }

    function getAliveTabs() {
        var now = new Date().getTime();

        return getTabs().filter(function (x) {
            return now - x.value.aliveAt < config.tabHeartbeatTimeout * 4;
        });
    }

    function getOtherAliveTabs() {
        return getAliveTabs().filter(function (x) {
            return x.key !== tabId;
        });
    }

    function cleanOldTabs() {
        var now = new Date().getTime();

        getTabs().filter(function (x) {
            return now - x.value.aliveAt >= config.tabHeartbeatTimeout * 4;
        }).forEach(function (x) {
            return removeItem$1(x.key);
        });
    }

    function isCurrentTabEventSource() {
        return isEventSourceTab;
    }

    function addIsEventSourceListener(listener) {
        isEventSourceListeners.push(listener);
    }

    var _createClass$1 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

    function _classCallCheck$2(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var AppEventSource = function () {
        function AppEventSource(url, configuration) {
            var _this = this;

            _classCallCheck$2(this, AppEventSource);

            this._url = url;
            this._configuration = configuration;

            this._sseListeners = [];
            this._onErrorEventListeners = [];

            this._sseChannel = new BroadcastChannel(config.sse.broadcastChannel);

            this._isEventSourceTab = isCurrentTabEventSource();

            if (this._isEventSourceTab) {
                console.log("Current tab must connect!"); // todo: delete
                this._connect();
            } else {
                console.log("Do not connect. Listen to other tab!"); // todo: delete
            }

            addIsEventSourceListener(function (isEventSourceTab) {
                _this._isEventSourceTab = isEventSourceTab;

                if (_this._isEventSourceTab) {
                    console.log("Current tab must connect!"); // todo: delete
                    _this._connect();
                } else {
                    console.log("Disconnect. listen to other tab"); // todo: delete
                    _this._closeEventSource();
                }
            });

            this.addConnectedEventListener(function (e, data) {
                return _this._onConnected(e, data);
            });

            this.addConnectedEventListener(function (e) {
                if (!_this._isEventSourceTab) return;
                _this._sseChannel.postMessage({ type: e.type, data: e.data });
            });

            this.addPostsListener(function (e) {
                if (!_this._isEventSourceTab) return;
                _this._sseChannel.postMessage({ type: e.type, data: e.data });
            });

            this.addBoardDescriptionListener(function (e) {
                if (!_this._isEventSourceTab) return;
                _this._sseChannel.postMessage({ type: e.type, data: e.data });
            });

            this.addErrorEventListener(function (e) {
                if (!_this._isEventSourceTab) return;
                _this._sseChannel.postMessage({ type: e.type, readyState: _this._eventSource.readyState });
            });

            // Listen to other tab.
            this._sseChannel.addEventListener('message', function (e) {
                return _this._sseChannelListener(e);
            });

            addEventListener('beforeunload', function (e) {
                return _this._close();
            });
        }

        _createClass$1(AppEventSource, [{
            key: "addErrorEventListener",
            value: function addErrorEventListener(listener) {
                this._onErrorEventListeners.push(listener);
            }
        }, {
            key: "addPostsListener",
            value: function addPostsListener(listener) {
                this._addJsonEventListener(config.sse.type.post, function (e, data) {
                    return listener(e, parsePost(data));
                });
            }
        }, {
            key: "addBoardDescriptionListener",
            value: function addBoardDescriptionListener(listener) {
                this._addJsonEventListener(config.sse.type.boardDescription, function (e, data) {
                    return listener(e, parseBoard(data));
                });
            }
        }, {
            key: "addConnectedEventListener",
            value: function addConnectedEventListener(listener) {
                if (!this._isEventSourceTab) listener({ type: config.sse.type.connected }, {});

                this._addJsonEventListener(config.sse.type.connected, listener);
            }
        }, {
            key: "_close",
            value: function _close() {
                this._sseChannel.close();
                this._closeEventSource();
            }
        }, {
            key: "_closeEventSource",
            value: function _closeEventSource() {
                if (this._eventSource) this._eventSource.close();
            }
        }, {
            key: "_connect",
            value: function _connect() {
                var _this2 = this;

                this._eventSource = new EventSource(this._url, this._configuration);
                this._eventSource.onerror = function (e) {
                    return _this2._onError(e);
                };

                this._sseListeners.forEach(function (sseListener) {
                    return _this2._restoreEventListener(sseListener.type, sseListener.listener);
                });
            }
        }, {
            key: "_onConnected",
            value: function _onConnected(e, data) {
                if (!this._isEventSourceTab) return;

                this._subscriptionId = data.subscriptionId;
                this._keepSubscriptionAlive();
                console.log(config.sse.type.connected, this._subscriptionId);
            }
        }, {
            key: "_keepSubscriptionAlive",
            value: function _keepSubscriptionAlive() {
                var _this3 = this;

                if (!this._isEventSourceTab) return;

                clearTimeout(this._keepAliveTimeoutId);

                if (this._eventSource.readyState !== EventSource.OPEN) return;

                console.log('keep alive', this._subscriptionId);

                this._keepAliveTimeoutId = setTimeout(function () {
                    continueSubscription$1(_this3._subscriptionId).then(function () {
                        return _this3._keepSubscriptionAlive();
                    });
                }, config.sse.continueSubscriptionTimeout);
            }
        }, {
            key: "_addJsonEventListener",
            value: function _addJsonEventListener(type, listener) {
                this._addEventListener(type, function (e) {
                    return listener(e, JSON.parse(e.data));
                });
            }
        }, {
            key: "_addEventListener",
            value: function _addEventListener(type, listener) {
                this._sseListeners.push(new SseListener(type, listener));
                if (this._eventSource) this._eventSource.addEventListener(type, listener);
            }
        }, {
            key: "_restoreEventListener",
            value: function _restoreEventListener(type, listener) {
                this._eventSource.addEventListener(type, listener);
            }
        }, {
            key: "_onError",
            value: function _onError(e) {
                var _this4 = this;

                console.log('Event Source failed, readyState = ', this._eventSource.readyState);

                this._onErrorEventListeners.forEach(function (listener) {
                    return listener(e, _this4._eventSource.readyState);
                });

                if (this._eventSource.readyState !== EventSource.CLOSED) return;

                // firefox won't try to reconnect after CLOSED state, so we have to do it ourselves
                clearTimeout(this._reconnectTimeoutId);
                this._reconnectTimeoutId = setTimeout(function () {
                    return _this4._connect();
                }, config.sse.reconnectTimeout);
            }
        }, {
            key: "_sseChannelListener",
            value: function _sseChannelListener(e) {
                if (this._isEventSourceTab) return;

                var sse = e.data;
                this._sseListeners.filter(function (sseListener) {
                    return sseListener.type === sse.type;
                }).forEach(function (sseListener) {
                    return sseListener.listener(sse, sse.data);
                });

                if (sse.type === 'error') {
                    this._onErrorEventListeners.forEach(function (listener) {
                        listener(sse, sse.readyState);
                    });
                }
            }
        }]);

        return AppEventSource;
    }();

    var SseListener = function SseListener(type, listener) {
        _classCallCheck$2(this, SseListener);

        this.type = type;
        this.listener = listener;
    };

    // file: repository.js

    function getRecentPosts$1() {
        return getRecentPosts().then(function (r) {
            return r.data;
        }).then(saveMany);
    }

    function getPostsBefore$1(postId) {
        return getPostsBefore(postId).then(function (r) {
            return r.data;
        }).then(saveMany);
    }

    function getPostsAfter$1(postId) {
        return getPostsAfter(postId).then(function (r) {
            return r.data;
        }).then(saveMany);
    }

    function getLogsPage$1(pageNumber) {
        return getLogsPage(pageNumber).then(function (r) {
            saveMany(r.data);
            return r;
        });
    }

    function getPost$1(postId) {
        var includeIndex = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;

        var cachedPost = get(postId);
        if (cachedPost && !includeIndex) return Promise.resolve(cachedPost);

        return getPost(postId, includeIndex).then(function (r) {
            return r.data;
        }).then(save);
    }

    function getThreadPage$1(threadId) {
        var pageNumber = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;

        // null means the last page

        return getThreadPage(threadId, pageNumber).then(function (r) {
            r.data.op.thread = r.data.thread;

            r.data.posts = r.data.posts.map(function (p) {
                p.thread = r.data.thread;
                return p;
            });
            saveForThreadSlice(r.data);
            return r;
        });
    }

    function getWarpPage$1() {
        var boardName = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : null;
        var pageNumber = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;

        return getWarpPage(boardName, pageNumber).then(function (r) {
            r.data.shellThreads = r.data.shellThreads.map(function (st) {
                st.thread.board = st.thread.board || r.data.board;
                st.op.thread = st.thread;

                st.posts = st.posts.map(function (p) {
                    p.thread = st.thread;
                    return p;
                });
                return st;
            });
            saveForWarpSlice(r.data);
            return r;
        });
    }

    function updateBoardDescription$1(board) {
        return updateBoardDescription(board).then(function (r) {
            return r.data;
        });
    }

    function updateHiddenByOp$1(updatedHiddenByOp) {
        return getOrCreateClientId().then(function (clientId) {
            updatedHiddenByOp.clientId = clientId;

            return updateHiddenByOp(updatedHiddenByOp).then(function (r) {
                return r.data;
            });
        });
    }

    function publish$1(newPost) {
        return publish(newPost).then(function (r) {
            return r.data;
        });
    }

    var eventSource = void 0;

    function subscribe() {
        eventSource = new AppEventSource(getSubscribeApiUrl());
    }

    function addErrorEventListener(listener) {
        return eventSource.addErrorEventListener(listener);
    }

    function addPostsListener(listener) {
        return eventSource.addPostsListener(function (e, post) {
            // ignore ephemeral posts
            if (!post.ephemeral) return;

            listener(e, save(post));
        });
    }

    function addConnectedEventListener(listener) {
        return eventSource.addConnectedEventListener(listener);
    }

    function continueSubscription$1(subscriptionId) {
        return continueSubscription(subscriptionId);
    }

    // file: PostSubmitter.js
    // done
    var PostSubmitter = function () {
        var that = {};

        /**
         * @param {Object} params - keys: boardName, subject, threadId,
         *                                onPartIsSent, timeout, onTimeout, onError
         */
        that.publishParts = function (parts, params) {
            return getOrCreateClientId().then(function (clientId) {
                var newPost = new NewPost(clientId, parts[0], params.threadId, params.subject, params.boardName);

                return promiseTimeout(publish$1(newPost), params.timeout, params.onTimeout).then(function (post) {
                    params.onPartIsSent(parts.length);

                    return post.thread.threadId;
                }).catch(function (e) {
                    console.log("Error submitting message");
                    params.onError(e);
                });
            });
        };

        return that;
    }();

    // file: ThumbnailView.js
    var ThumbnailView = function () {
        var that = {};

        function create(url, className, src) {
            var imgE = document.createElement('img');
            imgE.alt = url;
            imgE.title = url;
            imgE.src = src;
            imgE.classList.add(className);
            imgE.referrerPolicy = "no-referrer";

            var pictureE = document.createElement('picture');
            pictureE.appendChild(imgE);
            return pictureE;
        }

        that.getYoutubeThumbnail = function (url) {
            var videoId = getYoutubeVideoId(url);
            if (!videoId) return null;
            return create(url, "youtube_thumb", 'https://img.youtube.com/vi/' + videoId + '/mqdefault.jpg');
        };

        function getYoutubeVideoId(url) {
            var regExp = /^https?:\/\/(?:www\.|m\.)?(?:youtube\.com|youtu\.be)\/(?:[\w\-]+\?v=|embed\/|v\/)?([\w\-]+)/;
            var match = url.match(regExp);
            return match && match[1].length === 11 ? match[1] : null;
        }

        that.getImgurThumbnail = function (url) {
            var regExp = /^.*imgur.com\/(\w{6,7})\w?\.(\w{3})/;
            var match = regExp.exec(url);
            if (!match) return null;
            var imageId = match[1];
            var imageExt = match[2];
            var thumbId = imageId + "m";
            return create(url, "imgur_thumb", 'https://i.imgur.com/' + thumbId + '.' + imageExt);
        };
        that.getTwitterThumbnail = function (url) {
            var regExp = /^(.*pbs.twimg.com\/media\/[\w_-]+)(?:\.(\w{3,4})|\?format=(\w{3,4}))/;
            var match = regExp.exec(url);
            if (!match) return null;
            return create(url, "twitter_thumb", match[1] + '.' + (match[2] || match[3]) + ':thumb');
        };

        that.getPloudfrThumbnailV = function (url) {
            var regExp = /^https?:\/\/(?:www\.)?video.ploud.fr\/videos\/(?:watch|embed)\/([\w\-]+)/;
            return getPeerTubeThumbnailV(url, 'video.ploud.fr', 'ploudfr_thumb', regExp);
        };

        that.getLibreVideoThumbnailV = function (url) {
            var regExp = /^https?:\/\/(?:www\.)?libre.video\/videos\/(?:watch|embed)\/([\w\-]+)/;
            return getPeerTubeThumbnailV(url, 'libre.video', 'librevideo_thumb', regExp);
        };

        that.getPeertubeSocialThumbnailV = function (url) {
            var regExp = /^https?:\/\/(?:www\.)?peertube.social\/videos\/(?:watch|embed)\/([\w\-]+)/;
            return getPeerTubeThumbnailV(url, 'peertube.social', 'peertubesocial_thumb', regExp);
        };

        function getPeerTubeThumbnailV(url, domain, thumbClass, regExp) {
            var match = url.match(regExp);
            if (!match) return null;
            var videoId = match[1];
            return create(url, thumbClass, 'https://' + domain + '/static/thumbnails/' + videoId + '.jpg');
        }

        return that;
    }();

    // file: EmbedView.js
    var EmbedView = function () {
        var that = {};

        function create(url, className, src) {
            var iframeE = document.createElement("iframe");
            iframeE.setAttribute('allowFullScreen', '');
            iframeE.classList.add(className);
            iframeE.src = src;
            iframeE.title = url;
            return iframeE;
        }

        that.getBitchuteEmbed = function (url) {
            var regExp = /^https?:\/\/(?:www\.)?bitchute.com\/(?:video|embed|torrent\/[^\/]+)\/([^\/?#&]+)/;
            var match = regExp.exec(url);
            if (!match) return null;

            var videoId = match[1];
            return create(url, "bitchute_thumb", 'https://www.bitchute.com/embed/' + videoId);
        };

        return that;
    }();

    // file: LinkView.js

    var LinkView = function () {
        var that = {};

        that.formatLink = function (url, name) {
            // fix url: replace &amp; back to &
            url = url.replace("&amp;", "&");

            var aE = document.createElement("a");
            aE.className = 'extlink';
            aE.target = '_blank';
            aE.href = url;
            aE.referrerPolicy = "no-referrer";
            aE.rel = "noreferrer";
            aE.textContent = name ? name : tryDecodeURI(url);

            function addThumb(thumbE, className) {
                aE.textContent = "";
                aE.className += " " + className;
                aE.appendChild(thumbE);
            }

            if (doEmbedBitchute()) {
                var _thumbE = EmbedView.getBitchuteEmbed(url);
                if (_thumbE) {
                    addThumb(_thumbE, "bitchute_link");
                    aE.appendChild(document.createElement("br"));
                    aE.appendText(tryDecodeURI(url));
                    return aE.outerHTML;
                }
            }

            if (!doShowImgs()) {
                return aE.outerHTML;
            }

            var thumbE = ThumbnailView.getYoutubeThumbnail(url);
            if (thumbE) {
                addThumb(thumbE, "youtube_link");
                return aE.outerHTML;
            }
            thumbE = ThumbnailView.getImgurThumbnail(url);
            if (thumbE) {
                addThumb(thumbE, "imgur_link");

                return aE.outerHTML;
            }
            thumbE = ThumbnailView.getTwitterThumbnail(url);
            if (thumbE) {
                addThumb(thumbE, "twitter_link");
                return aE.outerHTML;
            }
            thumbE = ThumbnailView.getPloudfrThumbnailV(url);
            if (thumbE) {
                addThumb(thumbE, "ploudfr_link");
                return aE.outerHTML;
            }
            thumbE = ThumbnailView.getLibreVideoThumbnailV(url);
            if (thumbE) {
                addThumb(thumbE, "librevideo_link");
                return aE.outerHTML;
            }
            thumbE = ThumbnailView.getPeertubeSocialThumbnailV(url);
            if (thumbE) {
                addThumb(thumbE, "peertubesocial_link");
                return aE.outerHTML;
            }
            return aE.outerHTML;
        };
        return that;
    }();

    // file: ChanPostBaseView.js
    // done
    var ChanPostBaseView = function ChanPostBaseView(isReplyDivNeeded) {
        var that = this;

        that.postDivE = document.createElement('div');
        that.postheadDivE = document.createElement("div");
        that.postIdSpanE = document.createElement("span");
        that.nameSpanE = document.createElement("span");
        that.postbodyDivE = document.createElement("div");
        that.hideByOpBtnE = document.createElement("button");
        that.unhideByOpBtnE = document.createElement("button");

        that.postheadDivE.appendChild(that.postIdSpanE);
        that.postheadDivE.appendChild(that.nameSpanE);
        that.postDivE.appendChild(that.hideByOpBtnE);
        that.postDivE.appendChild(that.unhideByOpBtnE);
        that.postDivE.appendChild(that.postheadDivE);
        that.postDivE.appendChild(that.postbodyDivE);

        if (isReplyDivNeeded) {
            that.replyDivE = document.createElement("div");
            that.postDivE.insertBefore(that.replyDivE, that.postbodyDivE);
        }

        that.appendTo = function (element) {
            element.appendChild(that.postDivE);
        };
    };

    var _createClass$2 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

    function _classCallCheck$3(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var NotificationV = function () {
        function NotificationV(notificationType, content, autoHideAfter) {
            var _this = this;

            _classCallCheck$3(this, NotificationV);

            if (!(notificationType in config.notification.type)) {
                throw new Error('Unsupported notification type: ' + notificationType);
            }

            var notificationE = document.createElement('div');

            this.element = notificationE;
            notificationE.notificationV = this;

            notificationE.classList.add('notification');
            notificationE.classList.add('notification_' + notificationType);

            var notificationContentE = document.createElement('div');
            notificationContentE.classList.add('notification_content');
            notificationContentE.appendAny(content);

            notificationE.appendChild(notificationContentE);

            var closeBtnE = document.createElement('button');
            closeBtnE.classList.add('close_btn');
            // closeBtnE.classList.add('icon-close');
            closeBtnE.appendText('x');

            notificationE.appendChild(closeBtnE);

            closeBtnE.addEventListener('click', function (e) {
                return _this._close();
            }, false);

            setTimeout(function () {
                return _this._close();
            }, autoHideAfter);
        }

        _createClass$2(NotificationV, [{
            key: '_close',
            value: function _close() {
                this.element.remove();
            }
        }]);

        return NotificationV;
    }();

    function showNotification(notificationType, content) {
        var autoHideAfter = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : config.notification.autoHideAfter;

        var notificationContainerE = $("#notification_container");

        var notificationV = new NotificationV(notificationType, content, autoHideAfter);
        notificationContainerE.appendChild(notificationV.element);
    }

    // file: ChanPostView.js

    var ChanPostView = function ChanPostView(post, isPostJoined, skipAddingId, isOp) {
        var that = this;
        var base = new ChanPostBaseView(isPostJoined);

        base.postDivE.className = "post";
        if (!skipAddingId) base.postDivE.id = 'p' + post.postId;
        base.postheadDivE.className = "posthead";
        base.postheadDivE.insertTextBefore(" ", base.nameSpanE);

        base.postIdSpanE.className = "post-id";
        base.postIdSpanE.style.color = post.fg.toHSL();
        base.postIdSpanE.style.backgroundColor = post.bg.toHSL();
        base.postIdSpanE.setAttribute("data-post-id", post.postId);
        base.postIdSpanE.appendText(toShortcutUuid(post.postId));

        base.nameSpanE.classList.add("name");
        base.nameSpanE.appendText(post.user.nickname);
        if (post.user.banned) base.nameSpanE.classList.add('banned');
        base.postbodyDivE.className = "postbody";

        base.postheadDivE.appendText(' : ' + formatDate(post.postDate));
        base.postbodyDivE.innerHTML = PostBodyView.getFormattedText(post.text);

        if (isPostJoined) {
            base.postheadDivE.hide();
            base.postDivE.classList.add("joined");
            base.postbodyDivE.classList.add("joined");

            base.replyDivE.classList.add("reply-to-post");
            base.replyDivE.appendText("");

            base.replyDivE.title = post.postId + ' - ' + formatDate(post.postDate);
        }

        base.hideByOpBtnE.classList.add("hide_by_op_btn");
        base.unhideByOpBtnE.classList.add("unhide_by_op_btn");
        base.hideByOpBtnE.title = 'Hide for all (you have to be OP)';
        base.unhideByOpBtnE.title = 'Unhide for all (you have to be OP)';

        function hideByOp() {
            base.postDivE.classList.add("hidden_by_op");
            base.hideByOpBtnE.hide();
            base.unhideByOpBtnE.show();
        }

        function unhideByOp() {
            base.postDivE.classList.remove("hidden_by_op");
            base.hideByOpBtnE.show();
            base.unhideByOpBtnE.hide();
        }

        if (post.hiddenByOp) {
            hideByOp();
        } else {
            unhideByOp();
        }

        if (isOp || post.thread.threadId === config.chat.threadId) {
            base.hideByOpBtnE.hide();
        }

        that.appendTo = base.appendTo;

        that.setOnClickPostIdEventHandler = function (eventHandler) {
            if (isPostJoined) {
                base.replyDivE.onclick = eventHandler;
            } else {
                base.postIdSpanE.onclick = eventHandler;
            }
        };

        var referenceSpansEs = base.postbodyDivE.getElementsByClassName("msgref");
        referenceSpansEs.forEach(function (referenceSpanE) {
            var postReferenceView = new PostReferenceView();
            postReferenceView.initialize(referenceSpanE);
            postReferenceView.setMouseEvents();
        });

        base.unhideByOpBtnE.addEventListener('click', function (e) {
            updateHiddenByOp$1(new UpdatedHiddenByOp(null, post.postId, false)).then(function (_) {
                unhideByOp();
                showNotification(config.notification.type.success, 'The post was unhidden.');
            }).catch(function (e) {
                if (e.message === "Invalid clientId") {
                    showNotification(config.notification.type.error, 'You\'re not OP of this thread: invalid clientId', 5000);
                } else {
                    showNotification(config.notification.type.error, 'Failed to unhide the post: ' + e.message);
                }
            });
        }, false);

        base.hideByOpBtnE.addEventListener('click', function (e) {
            updateHiddenByOp$1(new UpdatedHiddenByOp(null, post.postId, true)).then(function (_) {
                hideByOp();
                showNotification(config.notification.type.success, 'The post was hidden.');
            }).catch(function (e) {
                if (e.message === "Invalid clientId") {
                    showNotification(config.notification.type.error, 'You\'re not OP of this thread: invalid clientId', 5000);
                } else {
                    showNotification(config.notification.type.error, 'Failed to hide the post: ' + e.message);
                }
            });
        }, false);
    };

    // file: hljs_service.js

    var serviceLoader$1 = new ServiceLoader(config.hljs.cssPath, config.hljs.scriptPath);

    function enableHljs(isEnabled) {
        return serviceLoader$1.enableService(isEnabled);
    }

    function highlightCode(element) {
        if (!serviceLoader$1.isServiceEnabled) {
            // undoHighlighting(element);
            return;
        }

        // show elements without highliting first, don't let highlighting slow displaying them
        setTimeout(function () {
            // :not(.hljs) is to prevent applying it more than once
            element.$a("code:not(.hljs)").forEach(function (codeE) {
                return hljs.highlightBlock(codeE);
            });
        }, 200);
    }

    // NOTE: it turns out that it is much slower that recreating html
    // function undoHighlighting(element) {
    //     element.$a("code.hljs").forEach(codeE => {
    //         codeE.classList.remove('hljs');
    //         const code = codeE.innerText;
    //         codeE.empty();
    //         codeE.appendText(code);
    //     });
    // }

    // file: PostReferenceView.js

    var PostReferenceView = function PostReferenceView() {
        var that = this;
        var span = void 0;
        var requestTimeout = 50; // ms
        var timeoutBeforeRemoving = 20; // ms

        that.create = function (postId) {
            span = document.createElement("span");
            span.className = "msgref";

            setPostId(postId);
            var link = document.createElement("a");
            link.href = getPostLinkUrl(postId);
            link.appendText("@" + toShortcutUuid(postId));
            span.appendChild(link);
        };

        that.initialize = function (element) {
            return span = element;
        };

        that.setMouseEvents = function () {
            function getQuotePreview(postId) {
                return document.querySelector(".quote-preview[data-post-id='" + postId + "']");
            }

            var timeoutId = void 0;

            span.onmouseenter = function (e) {
                var postId = getPostId();

                if (getQuotePreview(postId)) {
                    return;
                }
                if (e.relatedTarget && e.relatedTarget.classList.contains("quote-preview")) {
                    return;
                }
                setWaitCursor();
                // console.log(e.target)

                var clientRect = span.getBoundingClientRect();

                var left = //(hasPageScrollbar()
                //?
                span.offsetLeft + span.offsetParent.offsetLeft
                //: clientRect.left)
                + clientRect.width / 2;

                var top = (hasPageScrollbar() ? span.offsetTop + span.offsetParent.offsetTop : clientRect.top) + clientRect.height / 2;

                timeoutId = setTimeout(function () {
                    showQuotePreview(postId, clientRect, left, top).then(resetWaitCursor, resetWaitCursor);
                }, requestTimeout);
            };

            span.onmouseleave = function (e) {
                clearTimeout(timeoutId);
                // if (e.relatedTarget.classList.contains(chan.classNames.quotePreview)) {
                //     return;
                // }
                var postId = getPostId();

                var quotePreview = getQuotePreview(postId);
                var relatedQuotePreview = e.relatedTarget.getClosestByClassName("quote-preview");
                if (quotePreview && relatedQuotePreview !== quotePreview) {
                    // console.log("remove", e, e.relatedTarget)
                    setTimeout(function () {
                        quotePreview.remove();
                    }, 10);
                }
                // chan.removeQuotePreviews();
            };

            removeQuotePreviewsOnClick();

            span.onclick = function (e) {
                var postId = getPostId();
                var hash = "#p" + postId;
                if (!document.querySelector(hash)) return;
                e.preventDefault();

                location.href = location.pathname + location.search + hash;
            };
        };

        function showQuotePreview(postId, referenceSpanClientRect, left, top) {
            var postPromise = getPost$1(postId);

            return postPromise.then(function (post) {
                var postView = new ChanPostView(post, false, true);

                var quotePreview = document.createElement("div");
                quotePreview.className = "quote-preview";
                quotePreview.setAttribute("data-post-id", postId);
                postView.appendTo(quotePreview);

                // find out the horizontal position
                if (left < document.body.clientWidth / 2) {
                    // quotePreview will be on the right
                    quotePreview.style.left = left + "px";
                    // console.log(Math.floor(document.body.clientWidth - left - 10));
                    quotePreview.style.width = Math.floor(document.body.clientWidth - left - 10) + "px";
                } else {
                    // on the left
                    quotePreview.style.right = document.body.clientWidth - left + "px";
                    quotePreview.style.width = Math.floor(left - 10) + "px";
                }

                // find out the vertical position
                quotePreview.style.top = 0; // prevent appearing of the scrollbar (on the chat page).

                // const ctr = document.getElementById("container");
                // ctr.style.width = getComputedStyle(ctr, null).width;

                document.body.appendChild(quotePreview); // quotePreview.clientHeight is unknown for a hidden element.
                if (referenceSpanClientRect.top < window.innerHeight / 2) {
                    // below
                    var quotePreviewHeight = getQuotePreviewHeight(quotePreview);
                    if (quotePreviewHeight > window.innerHeight - top) {
                        var postBody = quotePreview.getElementsByClassName("postbody")[0];

                        var height = Math.floor(postBody.offsetHeight - quotePreviewHeight + window.innerHeight - top);

                        postBody.style.height = height + "px";
                    }
                    quotePreview.style.top = top + "px";
                    // console.log(quotePreviewHeight, window.innerHeight - top);
                    // console.log("top=",referenceSpanClientRect)
                } else {
                    // above
                    var _quotePreviewHeight = getQuotePreviewHeight(quotePreview);
                    if (_quotePreviewHeight > top) {
                        var _postBody = quotePreview.getElementsByClassName("postbody")[0];

                        var _height = Math.floor(_postBody.offsetHeight - _quotePreviewHeight + top);

                        _postBody.style.height = _height + "px";
                    }
                    quotePreview.style.top = top + 2 - quotePreview.clientHeight - referenceSpanClientRect.height + "px";
                    // console.log(referenceSpanClientRect.top);
                }
                // document.body.appendChild(quotePreview);
                quotePreview.onmouseleave = function (e) {
                    if (!e.relatedTarget) return;
                    var relatedQuotePreview = e.relatedTarget.getClosestByClassName("quote-preview");
                    var quotePreviews = document.getElementsByClassName("quote-preview");

                    var indexOfThisQuotePreview = quotePreviews.indexOf(e.target);
                    var indexOfRelatedQuotePreview = quotePreviews.indexOf(relatedQuotePreview);

                    if (indexOfRelatedQuotePreview > indexOfThisQuotePreview || e.relatedTarget.classList.contains("msgref")) {
                        return;
                    }
                    // console.log("asdf", e.relatedTarget, indexOfRelatedQuotePreview, indexOfThisQuotePreview);

                    setTimeout(function () {
                        return e.target.remove();
                    }, timeoutBeforeRemoving);

                    // const ctr = document.getElementById("container");
                    // ctr.style.width = "";
                };

                if (doHighlightCode()) {
                    highlightCode(quotePreview);
                }
            }).catch(function (errorResponse) {
                showNotification(config.notification.type.error, errorResponse.message);
            });
        }

        var existingOnClickEventHandler = document.body.onclick || function (x) {
            return x;
        };

        function removeQuotePreviewsOnClick() {
            document.body.onclick = function (e) {
                existingOnClickEventHandler(e);

                if (e.target.getClosestByClassName("quote-preview") || e.target.classList.contains("msgref")) {
                    return;
                }
                removeQuotePreviews();
            };
        }

        function removeQuotePreviews(e) {
            // http://stackoverflow.com/a/23989142
            var quotePreviews = document.querySelectorAll(".quote-preview");

            quotePreviews.forEach(function (quotePreview) {
                return quotePreview.remove();
            });

            // const ctr = document.getElementById("container");
            // ctr.style.width = "";
        }

        function getPostId() {
            return span.getAttribute("data-post-id");
        }

        function setPostId(postId) {
            span.setAttribute("data-post-id", postId);
        }

        that.getOuterHtml = function () {
            return span.outerHTML;
        };

        function hasPageScrollbar() {
            return document.body.offsetHeight > innerHeight;
        }

        function getQuotePreviewHeight(quotePreview) {
            var quotePreviewStyle = getComputedStyle(quotePreview);
            var quotePreviewHeight = quotePreview.offsetHeight + parseFloat(quotePreviewStyle.marginTop.slice(0, -2));
            // + parseFloat(quotePreviewStyle.paddingTop.slice(0, -2))
            // + parseFloat(quotePreviewStyle.marginTop.slice(0, -2))
            // + parseFloat(quotePreviewStyle.paddingBottom.slice(0, -2));
            return quotePreviewHeight;
        }

        function setWaitCursor() {
            var link = span.firstChild;
            link.style.cursor = "wait";
        }

        function resetWaitCursor() {
            var link = span.firstChild;
            link.style.cursor = "";
        }
    };

    // file: PostBodyView.js

    var PostBodyView = function (insertQuotes, splitLongText) {
        var that = {};
        var urlRe = /^(?:(?:(?:https?|ftp):\/\/)(?:\S+(?::\S*)?@)?(?:(?!10(?:\.\d{1,3}){3})(?!127(?:\.\d{1,3}){3})(?!169\.254(?:\.\d{1,3}){2})(?!192\.168(?:\.\d{1,3}){2})(?!172\.(?:1[6-9]|2\d|3[0-1])(?:\.\d{1,3}){2})(?:[1-9]\d?|1\d\d|2[01]\d|22[0-3])(?:\.(?:1?\d{1,2}|2[0-4]\d|25[0-5])){2}(?:\.(?:[1-9]\d?|1\d\d|2[0-4]\d|25[0-4]))|(?:(?:[a-z\u00a1-\uffff0-9]+-?)*[a-z\u00a1-\uffff0-9]+)(?:\.(?:[a-z\u00a1-\uffff0-9]+-?)*[a-z\u00a1-\uffff0-9]+)*(?:\.(?:[a-z\u00a1-\uffff0-9]{2,})))(?::\d{2,5})?(?:\/[^\s]*)?|magnet:\?(?:[^\s\]\)]+))/;

        that.getFormattedText = function (text) {
            function escapeHtml(unsafe) {
                return unsafe.replace(/&/g, "&amp;").replace(/</g, "&lt;").replace(/>/g, "&gt;").replace(/"/g, "&quot;").replace(/'/g, "&#039;");
            }

            text = escapeHtml(text);

            text = text.replace(/\n{3,}/g, "\n\n");

            var formattedText = insertQuotes(text);
            formattedText = formatText(formattedText);

            return formattedText;
        };

        // it is used for the forms
        that.getParts = function (text) {
            return splitLongText(text);
        };

        that.getPostIdText = function (postId) {
            return '@' + toSimpleUuid(postId) + ' ';
        };

        function formatText(text) {
            for (var i = 0; i < text.length; i++) {
                var textToParse = text.slice(i);
                var markupFreePrefix = i > 0 ? text.substring(0, i) : "";

                var newline = parserNewline(textToParse);
                if (newline) {
                    return markupFreePrefix + '<br>' + formatText(newline.rest);
                }
                var bold = parseBold(textToParse);
                if (bold) {
                    return markupFreePrefix + '<b>' + formatText(bold.match) + '</b>' + formatText(bold.rest);
                }
                var italic = parseItalic(textToParse);
                if (italic) {
                    return markupFreePrefix + '<i>' + formatText(italic.match) + '</i>' + formatText(italic.rest);
                }
                var code = parseCode(textToParse);
                if (code) {
                    return markupFreePrefix + '<pre><code class=\'' + code.lang + '\'>' + code.match + '</code></pre>' + formatText(code.rest);
                }
                var mono = parseMono(textToParse);
                if (mono) {
                    return markupFreePrefix + '<samp>' + formatText(mono.match) + '</samp>' + formatText(mono.rest);
                }
                var strike = parseStrike(textToParse);
                if (strike) {
                    return markupFreePrefix + '<s>' + formatText(strike.match) + '</s>' + formatText(strike.rest);
                }
                var tex = parseTeX(textToParse);
                if (tex) {
                    var katexElement = document.createElement("div");
                    renderKatex(tex.match, katexElement);

                    return markupFreePrefix + katexElement.innerHTML + formatText(tex.rest);
                }
                var spoiler = parseSpoiler(textToParse);
                if (spoiler) {
                    return markupFreePrefix + '<span class="spoiler">' + formatText(spoiler.match) + '</span>' + formatText(spoiler.rest);
                }
                var namedLink = parseNamedLink(textToParse);
                if (namedLink) {
                    return markupFreePrefix + LinkView.formatLink(namedLink.match, namedLink.name) + formatText(namedLink.rest);
                }
                var link = parseLink(textToParse);
                if (link) {
                    return markupFreePrefix + LinkView.formatLink(link.match) + formatText(link.rest);
                }
                var postReference = parsePostReference(textToParse);
                if (postReference) {
                    var postReferenceView = new PostReferenceView();
                    postReferenceView.create(postReference.match);

                    return markupFreePrefix + postReferenceView.getOuterHtml() + formatText(postReference.rest);
                }
            }
            return text;
        }

        function parseNamedLink(text) {
            var match = /^\[([^\n\r\]]+)\]\(([^\)]+)\)/.exec(text);
            if (!match) return null;
            var name = match[1];
            // const urlMatch = urlRe.exec(match[2]);
            // if (!urlMatch) return null;
            // const url = urlMatch[0];
            var url = match[2];
            var lengthToSkip = match[0].length;
            return { match: url, name: name, rest: text.slice(lengthToSkip) };
        }

        function parseLink(text) {
            var match = urlRe.exec(text);
            if (!match) return null;
            var url = match[0];
            return { match: url, rest: text.slice(url.length) };
        }

        function parserNewline(text) {
            if (text[0] === '\n') {
                return { rest: text.slice(1) };
            }
            return null;
        }

        function parseBold(text) {
            return parseSimpleMarkup("**", text);
        }

        function parseItalic(text) {
            return parseSimpleMarkup("//", text);
        }

        function parseMono(text) {
            return parseSimpleMarkup("`", text);
        }

        function parseTeX(text) {
            return parseSimpleMarkup("$", text);
        }

        function parseStrike(text) {
            return parseSimpleMarkup("~~", text);
        }

        function parseSpoiler(text) {
            return parseSimpleMarkup("%%", text);
        }

        function parseCode(text) {
            var code = parseSimpleMarkup("```", text);
            if (!code) return code;
            var langMatch = /^[^\s]*/.exec(code.match);
            if (!langMatch) return code;
            code.lang = langMatch[0];
            code.match = code.match.slice(code.lang.length).replace(/^\s/, "");
            return code;
        }

        function parseSimpleMarkup(markup, text) {
            if (!text.startsWith(markup)) return null;
            var idx = text.indexOf(markup, markup.length);
            if (idx < 0) return null;
            var match = text.substring(markup.length, idx);
            return { match: match, rest: text.slice(idx + markup.length) };
        }

        return that;
    }(function insertQuotes(text, level) {
        level = level || 0;

        var hiddenQuoteSign = function () {
            var hiddenQuoteSignSpan = document.createElement("span");
            hiddenQuoteSignSpan.className = "hidden-quote-sign";
            hiddenQuoteSignSpan.appendText(">");
            return hiddenQuoteSignSpan.outerHTML;
        }();

        return text.replace(/^(?:@[0-9a-f]+)?((?:\s*&gt;.+\n?)+)/gm, function (quote) {
            var postReference = "";
            quote = quote.replace(/^(@[0-9a-f]+)?\s*(&gt;\s*)/gm, function (_, p1) {
                postReference = p1 || "";
                return "";
            });

            quote = level > 10 ? quote : insertQuotes(quote, level + 1);

            var blockquote = document.createElement("blockquote");
            blockquote.innerHTML = quote + " "; // hotfix. todo: refactor

            return postReference + blockquote.outerHTML;
        });
    }, function splitLongText(text) {
        var partLength = config.maxPostLength;
        var words = text.split(/(?=\s)/g);
        var parts = [];

        var splitWordRe = new RegExp("(.|[\\r\\n]){1," + partLength + "}", "g");

        function splitLongWord(word) {
            return word.match(splitWordRe);
        }

        var j = 0;
        parts[0] = "";

        var partsLengthSum = 0;
        for (var i = 0; i < words.length; i++) {
            if (parts[j].length + words[i].length <= partLength) {
                parts[j] += words[i];
                continue;
            }

            partsLengthSum += parts[j].length;

            if (words[i].length <= partLength) {
                j++;
                parts[j] = words[i];
            } else {
                var subwords = splitLongWord(words[i]);
                j += subwords.length;
                parts = parts.concat(subwords);
            }
        }
        return parts;
    });

    // file: AbstractPostFormView.js

    var instance = void 0;
    var AbstractPostFormView = function AbstractPostFormView(publishParts) {
        var that = this;
        if (instance) {
            throw new Error("There cannot be more than one AbstractPostFormView");
        }
        instance = that;
        AbstractPostFormView.getInstance = function () {
            if (!instance) {
                throw new Error("There is no AbstractPostFormView instance");
            }
            return instance;
        };

        if (!publishParts) throw new Error("`publishParts` must be set");

        that.BasePostFormView = function (textboxView) {
            var base = this;
            if (!textboxView) throw new Error("textboxView must be set");

            base.publishParts = function (parts, params) {
                params.timeout = getSubmissionTimeout();
                params.onTimeout = onSubmissionTimeout;
                params.onError = onSubmissionFailure;

                publishParts(parts, params);
            };

            base.insertPostId = function (postId) {
                var postIdText = PostBodyView.getPostIdText(postId);
                textboxView.insertText(postIdText);
            };

            base.insertText = function (text) {
                textboxView.insertText(text);
            };

            base.getTextBeforeSelectionEnd = textboxView.getTextBeforeSelectionEnd;

            base.getOnPartIsSentFunction = function (numberOfParts) {
                textboxView.clear();

                return function onPartIsSent(currentNumberOfParts) {
                    var numberOfPartsToBeSent = currentNumberOfParts - 1;
                    if (numberOfPartsToBeSent === 0) {
                        textboxView.clearPlaceholder();
                        return;
                    }
                    var numberOfSentParts = numberOfParts - numberOfPartsToBeSent;
                    textboxView.setPlaceholder(numberOfSentParts + '/' + numberOfParts + ' is sent');
                };
            };

            function onSubmissionTimeout() {
                textboxView.setPlaceholder(getSubmissionTimeoutMsg());
            }

            function onSubmissionFailure(e) {
                if (e.errorCode === 'E0401') {
                    showNotification(config.notification.type.error, e.message + '. Please log in.');
                    return;
                }

                textboxView.setPlaceholder(getSubmissionFailureMsg());
            }
        };
    };

    // file: TitleView.js

    var TitleView = function () {
        var that = {};
        var postfix = "";

        that.set = function (text) {
            document.title = text + postfix;
        };

        that.add = function (text) {
            document.title += text;
        };

        return that;
    }();

    // file: TypingLoggerView.js
    // done
    var TypingLoggerView = function () {
        var that = {};
        var myTypingLog = [];

        that.runTypingLogger = function (textboxView) {
            var isInput = textboxView.getTagName() === "input";
            var isTextarea = textboxView.getTagName() === "textarea";
            if (!isInput && !isTextarea) return;
            var position = 0;

            // removes repeated messages in the log.
            function removeRepeated() {
                if (myTypingLog.length >= 2 && myTypingLog[myTypingLog.length - 1] === myTypingLog[myTypingLog.length - 2]) {

                    myTypingLog.pop();
                    //console.log("repeated", position, myTypingLog);
                }
            }

            textboxView.addOnKeyDownEventHandler(function (e) {
                // up arrow (only for input) or alt+i
                if (e.keyCode === 38 && isInput || e.altKey && e.keyCode === 73) {
                    if (position <= myTypingLog.length) {
                        position++;
                    }
                    //console.log("up", position, myTypingLog);
                }
                // down arrow (only for input) or alt+k
                else if (e.keyCode === 40 && isInput || e.altKey && e.keyCode === 75) {
                        if (position > 0) {
                            position--;
                        }
                        //console.log("down", position, myTypingLog);
                    } else {
                        return true;
                    }

                removeRepeated();

                var idx = myTypingLog.length - position;

                textboxView.setValue(idx < 0 || position < 1 ? "" : myTypingLog[idx]);

                return false;
            });

            function refreshLog(e) {
                var idx = myTypingLog.length - position;
                if ((position === 0 || idx === -1) && textboxView.getValue() !== 0) {
                    myTypingLog.push(textboxView.getValue());
                    position = 1;
                    //console.log("printable key up", position, myTypingLog);
                } else if (position === 1 && myTypingLog.length !== 0) {
                    // if not "enter" was pressed
                    if (e.keyCode !== 13) {
                        myTypingLog[idx] = textboxView.getValue();
                        if (textboxView.getValue().length === 0) {
                            myTypingLog.pop();
                            position = 0;
                            //console.log(myTypingLog, e.keyCode);
                        }
                    } else {
                        position = 0;
                        removeRepeated();
                    }
                    //console.log("update first", position, myTypingLog);
                } else if (position > 1 && idx >= 0 && myTypingLog[idx] !== textboxView.getValue()) {
                    myTypingLog.push(textboxView.getValue());
                    position = 1;
                    //console.log("update inside", position, myTypingLog);
                }
                return true;
            }

            textboxView.setOnKeyUpEventHandler(refreshLog);

            // after we inserted a message number, the input gets focused
            textboxView.setOnFocusEventHandler(refreshLog);

            // these two calls are needed for the case when we change <textarea> to <input>
            refreshLog({});
            removeRepeated();
        };

        return that;
    }();

    // file: TextboxView.js
    // done
    var TextboxView = function TextboxView(base) {
        var that = this;

        base.textbox.setAttribute("type", "text");

        that.setName = function (name) {
            return base.textbox.name = name;
        };
        that.setRequired = function () {
            return base.textbox.required = true;
        };
        that.setId = function (id) {
            return base.textbox.id = id;
        };
        that.getValue = function () {
            return base.textbox.value.trim();
        };
        that.setValue = function (value) {
            return base.textbox.value = value;
        };
        that.focus = function () {
            return base.textbox.focus();
        };
        that.getTagName = function () {
            return base.textbox.tagName.toLowerCase();
        };
        that.setPlaceholder = function (value) {
            return base.textbox.placeholder = value;
        };
        that.clearPlaceholder = function () {
            return base.textbox.placeholder = "";
        };

        that.setPlaceholderTemporarily = function (value, timeout) {
            that.setPlaceholder(value);
            setTimeout(function () {
                return that.clearPlaceholder();
            }, timeout);
        };

        that.setMaxLength = function (maxLength) {
            return base.textbox.maxLength = maxLength;
        };

        var onAfterAutoresize = function onAfterAutoresize() {};

        that.setOnAfterAutoresizeEventHandler = function (eventHandler) {
            onAfterAutoresize = eventHandler;
        };

        function resize() {
            var tbx = base.textbox;
            var parent = tbx.parentNode;

            // prevent page jerking via fixing the height of the parent node
            var originalParentHeight = parent.style.height;
            parent.style.height = getComputedStyle(parent).height;

            if (tbx.scrollHeight === tbx.clientHeight) tbx.style.height = "";

            var newHeight = tbx.scrollHeight + tbx.offsetHeight - tbx.clientHeight + "px";

            tbx.style.height = newHeight;

            // now release the parent
            parent.style.height = originalParentHeight;

            onAfterAutoresize();
        }

        var isAutoresizable = false;
        that.autoresize = function () {
            isAutoresizable = true;
            base.textbox.addEventListener("input", resize);
        };
        that.enableAutofocus = function () {
            return base.textbox.autofocus = true;
        };
        that.disableAutocomplete = function () {
            return base.textbox.autocomplete = "off";
        };
        that.addOnKeyDownEventHandler = function (eventHandler) {
            base.textbox.addEventListener("keydown", eventHandler);
        };
        that.setOnKeyUpEventHandler = function (eventHandler) {
            base.textbox.onkeyup = eventHandler;
        };
        that.setOnFocusEventHandler = function (eventHandler) {
            base.textbox.onfocus = eventHandler;
        };

        that.clear = function () {
            base.textbox.value = "";
            if (isAutoresizable) resize();
        };

        that.insertText = function (text) {
            var oldText = base.textbox.value;
            var selectionEnd = base.textbox.selectionEnd;

            var firstPart = oldText.slice(0, base.textbox.selectionEnd);
            var secondPart = oldText.slice(base.textbox.selectionEnd);
            base.textbox.value = firstPart + text + secondPart;

            base.textbox.selectionEnd = selectionEnd + text.length;
            base.textbox.focus();
            if (isAutoresizable) resize();
        };

        that.getTextBeforeSelectionEnd = function () {
            var tbxE = base.textbox;
            return tbxE.value.slice(0, tbxE.selectionEnd);
        };

        that.appendTo = base.appendTo;
        that.remove = base.remove;
    };

    // file: TextboxBaseView.js
    // done
    var TextboxBaseView = function TextboxBaseView(tagName, existingElement) {
        var that = this;

        that.textbox = !tagName && existingElement ? existingElement : document.createElement(tagName);

        that.appendTo = function (element) {
            element.appendChild(that.textbox);
        };
        that.remove = function () {
            that.textbox.remove();
        };
    };

    // file: shortcut_service.js
    function isEscape(event) {
        return event.key === "Escape" || event.keyCode === 27;
    }

    function isEnter(event) {
        return event.key === "Enter" || event.keyCode === 13;
    }

    function isCtrlEnter(event) {
        return isEnter(event) && event.ctrlKey;
    }

    function isAltEnter(event) {
        return isEnter(event) && event.altKey;
    }

    function isShiftEnter(event) {
        return isEnter(event) && event.shiftKey;
    }

    // file: ChatPostFormView.js
    // done
    var instance$1 = void 0;
    var ChatPostFormView = function ChatPostFormView() {
        var that = this;
        if (instance$1) {
            throw new Error("There cannot be more than one 'chat post' form");
        }
        instance$1 = that;
        ChatPostFormView.getInstance = function () {
            if (!instance$1) {
                throw new Error("There is no 'chat post' form instance");
            }
            return instance$1;
        };

        var chatPostForm = document.getElementById("chat-post-form");
        var postTextbox = document.getElementById("msgtxt");

        var postTextboxBaseView = new TextboxBaseView(null, postTextbox);
        var postTextboxView = new TextboxView(postTextboxBaseView);

        var abstractPostFormView = AbstractPostFormView.getInstance();
        var basePostFormView = new abstractPostFormView.BasePostFormView(postTextboxView);

        that.insertPostId = basePostFormView.insertPostId;
        that.insertText = basePostFormView.insertText;
        that.getTextBeforeSelectionEnd = basePostFormView.getTextBeforeSelectionEnd;

        var hideColorRe = new RegExp('^\\/hide\\s*@' + uuidPattern);
        var unhideColorRe = new RegExp('^\\/unhide\\s*@' + uuidPattern);

        function onSubmit() {
            try {
                var text = postTextboxView.getValue();
                if (!text) return;

                if (text === "/change-colors" || text === "/cc") {
                    updateClientId();
                    postTextboxView.clear();
                    postTextboxView.setPlaceholder("Your colors were changed.");
                    return;
                }
                if (text === "/help") {
                    document.getElementById("helpButton").click();
                    postTextboxView.clear();
                    return;
                }
                if (text === "/settings") {
                    document.getElementById("settingsButton").click();
                    postTextboxView.clear();
                    return;
                }
                var hideColorMatch = hideColorRe.exec(text);
                if (hideColorMatch) {
                    var postId = hideColorMatch[1];
                    hideColor(postId);
                    postTextboxView.clear();
                    return;
                }

                var unhideColorMatch = unhideColorRe.exec(text);
                if (unhideColorMatch) {
                    var _postId = unhideColorMatch[1];
                    unhideColor(_postId);
                    postTextboxView.clear();
                    return;
                }

                var boardMatch = /^\/([a-z]{1,3}) /.exec(text);
                if (boardMatch) {
                    postTextboxView.clear();
                    var boardName = boardMatch[1];
                    var boardDescription = text.substr(boardName.length + 2);

                    var board = new Board(boardName, boardDescription);

                    updateBoardDescription$1(board).then(function (response) {
                        // console.log(response);
                    });
                } else {
                    publish();
                }
            } catch (err) {
                console.log(err);
                return false;
            }
            return false;
        }

        var didIPost = false;

        function publish() {
            var boardName = config.chat.boardName;
            var threadId = config.chat.threadId;
            var subject = null;

            var text = postTextboxView.getValue();
            var postText = text;

            var reObj = /^\/([a-z]{1,3})\/([\-0-9a-fA-F]+) /.exec(text);
            if (reObj) {
                threadId = parseUuid(reObj[2]);

                if (threadId) {
                    threadId = threadId.match;
                    boardName = reObj[1];
                    postText = text.substr(('/' + boardName + '/' + threadId + ' ').length);
                }
            }

            reObj = /^\/([a-z]{1,3})\[(.+)\] /.exec(text);
            if (reObj) {
                threadId = null;
                boardName = reObj[1];
                subject = reObj[2];

                postText = text.substr(('/' + boardName + '[' + subject + '] ').length);
            }

            var parts = PostBodyView.getParts(postText);
            postTextboxView.clear();
            if (parts.length === 1 && parts[0] === "") return false;

            var onPartIsSent = basePostFormView.getOnPartIsSentFunction(parts.length);

            basePostFormView.publishParts(parts, {
                boardName: boardName,
                subject: subject,
                threadId: threadId,
                onPartIsSent: onPartIsSent
            });
            didIPost = true;

            return false;
        }

        function onKeyDown(e) {
            if (isAltEnter(e)) {
                postTextboxView.insertText("\n");
            } else if (isEnter(e) && !isShiftEnter(e)) {
                e.preventDefault();
                onSubmit();
            }
        }

        that.onNewPostsReceived = function () {
            if (didIPost) {
                didIPost = false;
                return;
            }
            TitleView.set("[new posts] - pfpmd chat");
        };

        function runTypingLogger() {
            TypingLoggerView.runTypingLogger(postTextboxView);
        }

        chatPostForm.onsubmit = onSubmit;

        postTextboxView.addOnKeyDownEventHandler(onKeyDown);
        postTextboxView.autoresize();

        that.setOnAfterAutoresizeEventHandler = function (eventHandler) {
            postTextboxView.setOnAfterAutoresizeEventHandler(eventHandler);
        };

        var onHideChange = function onHideChange() {};
        that.setFnOnHideChange = function (fn) {
            onHideChange = fn;
        };

        function hideColor(postId) {
            getPost$1(postId).then(function (post) {
                var filters = parseColorsFilters(getColorsFilters());

                if (!filters.some(function (_ref) {
                    var bg = _ref.bg,
                        fg = _ref.fg;
                    return post.bg === bg && post.fg === fg;
                })) {
                    filters.push({ bg: post.bg, fg: post.fg });
                }

                setColorsFilters(filters.map(function (_ref2) {
                    var bg = _ref2.bg,
                        fg = _ref2.fg;
                    return bg + ':' + fg + '\n';
                }).join(''));
                onHideChange();
            }).catch(function (errorResponse) {
                showNotification(config.notification.type.error, errorResponse.message);
            });
        }

        function unhideColor(postId) {
            getPost$1(postId).then(function (post) {
                var filters = parseColorsFilters(getColorsFilters());

                setColorsFilters(filters.filter(function (_ref3) {
                    var bg = _ref3.bg,
                        fg = _ref3.fg;
                    return !(bg === post.bg && fg === post.fg);
                }).map(function (_ref4) {
                    var bg = _ref4.bg,
                        fg = _ref4.fg;
                    return bg + ':' + fg + '\n';
                }).join(''));
                onHideChange();
            }).catch(function (errorResponse) {
                showNotification(config.notification.type.error, errorResponse.message);
            });
        }

        runTypingLogger();
    };

    // file: PostInfoBaseView.js
    // done
    var PostInfoBaseView = function PostInfoBaseView() {
        var that = this;

        that.postInfoSpan = document.createElement("span");
        that.boardLink = document.createElement("a");
        that.threadLink = document.createElement("a");

        that.postInfoSpan.appendChild(that.boardLink);
        that.postInfoSpan.appendChild(that.threadLink);

        that.appendTo = function (element) {
            element.appendChild(that.postInfoSpan);
        };
    };

    // file: ChatPostBaseView.js

    var ChatPostBaseView = function ChatPostBaseView() {
        var that = this;

        that.postDivE = document.createElement('div');
        that.postLeftE = document.createElement('div');
        that.postDivE.appendChild(that.postLeftE);

        that.infoIdE = document.createElement('div');
        that.infoIdE.classList.add('info-id');
        that.postLeftE.appendChild(that.infoIdE);

        that.infoNameE = document.createElement('div');
        that.infoNameE.classList.add('info-name');
        that.postLeftE.appendChild(that.infoNameE);

        that.postIdSpanE = document.createElement("span");
        that.infoIdE.appendChild(that.postIdSpanE);

        that.nicknameE = document.createElement("span");
        that.nicknameE.classList.add("nickname");
        that.infoNameE.appendChild(that.nicknameE);

        that.postbodyDivE = document.createElement("div");

        that.postInfoBaseView = new PostInfoBaseView();
        that.postInfoBaseView.appendTo(that.postbodyDivE);

        that.postDivE.appendChild(that.postbodyDivE);

        that.appendTo = function (element) {
            element.appendChild(that.postDivE);
        };

        that.insertBefore = function (element) {
            element.parentNode.insertBefore(that.postDivE, element);
        };
    };

    // file: PostInfoView.js
    // done
    var PostInfoView = function PostInfoView(base, boardName, threadId) {

        base.postInfoSpan.className = "post-info";

        base.postInfoSpan.insertTextBefore(" /", base.boardLink);
        base.postInfoSpan.insertTextBefore("/", base.threadLink);
        base.postInfoSpan.appendText("/");

        base.boardLink.appendText(boardName);
        base.boardLink.href = getBoardUrl(boardName);

        base.threadLink.appendText(toShortcutUuid(threadId));
        base.threadLink.href = getThreadUrl(boardName, threadId);
    };

    // file: ChatPostView.js
    // done
    var ChatPostView = function ChatPostView(post, base) {
        var that = this;
        base = base || new ChatPostBaseView();

        base.postDivE.className = "msg";
        base.postLeftE.classList.add('post-left');
        base.postbodyDivE.classList.add('post-body');

        var bodyE = document.createElement('div');
        bodyE.innerHTML = PostBodyView.getFormattedText(post.text);
        base.postbodyDivE.appendChild(bodyE);

        var boardName = post.thread.board.name;
        if (boardName !== config.chat.boardName) {
            var postInfoView = new PostInfoView(base.postInfoBaseView, boardName, post.thread.threadId);
        }

        base.postIdSpanE.className = "id";
        base.postIdSpanE.title = formatDate(post.postDate);
        base.postIdSpanE.style.color = post.fg.toHSL();
        base.postIdSpanE.style.backgroundColor = post.bg.toHSL();
        base.postIdSpanE.appendText(toShortcutUuid(post.postId));
        base.postIdSpanE.setAttribute("data-post-id", post.postId);

        base.nicknameE.appendText(post.user.nickname);

        that.getPost = function () {
            return post;
        };
        that.appendTo = base.appendTo;
        that.insertBefore = base.insertBefore;

        that.setOnClickPostIdEventHandler = function (eventHandler) {
            base.postIdSpanE.onclick = eventHandler;
        };

        var referenceSpans = base.postbodyDivE.getElementsByClassName("msgref");
        var postReferences = referenceSpans.forEach(function (referenceSpan) {
            var postReferenceView = new PostReferenceView();
            postReferenceView.initialize(referenceSpan);
            postReferenceView.setMouseEvents();
        });
    };

    // file: DaysSeparatorBaseView.js
    // done
    var DaysSeparatorBaseView = function DaysSeparatorBaseView() {
        var that = this;

        that.daysSeparatorDiv = document.createElement("div");
        that.dateSpan = document.createElement("span");

        that.daysSeparatorDiv.appendChild(that.dateSpan);

        that.appendTo = function (element) {
            element.appendChild(that.daysSeparatorDiv);
        };

        that.insertBefore = function (element) {
            element.parentNode.insertBefore(that.daysSeparatorDiv, element);
        };
    };

    // file: DaysSeparatorView.js
    // done
    var DaysSeparatorView = function DaysSeparatorView(date) {
        var that = this;
        var base = new DaysSeparatorBaseView();

        base.daysSeparatorDiv.className = "datesep";
        base.dateSpan.className = "date";

        base.dateSpan.appendText(formatDate());

        function formatDate() {
            var monthNames = ["January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December"];

            return date.getDate() + " " + monthNames[date.getMonth()] + " " + date.getFullYear();
        }

        this.appendTo = base.appendTo;
        that.insertBefore = base.insertBefore;
    };

    // file: ImageFailView.js
    // done
    var ImageFailView = function ImageFailView(text) {
        var that = this;
        var imageFailStrong = document.createElement("strong");
        imageFailStrong.className = "img-fail";

        imageFailStrong.appendText(text);

        that.appendTo = function (element) {
            element.appendChild(imageFailStrong);
        };
    };

    // file: ImageView.js
    // done
    var ImageView = function ImageView(maxContentLength, maxImgHeight) {

        var onImageLoadEventHandler = void 0;
        this.setOnImageLoadEventHandler = function (eventHandler) {
            onImageLoadEventHandler = eventHandler;
        };

        this.insertImage = function (link) {
            var params = getParamsForImageXhr(link);
            var rnd = Math.random();

            params.onLoadEventName = 'onImageHeadersLoad' + params.url + rnd;
            params.onErrorEventName = 'onImageHeadersLoadError' + params.url + rnd;

            var eventForGmXhr = new CustomEvent("eventForGmXhr", {
                'detail': params
            });
            window.dispatchEvent(eventForGmXhr);

            addEventListener(params.onLoadEventName, function (e) {
                params.onload(e.detail);
            });
            addEventListener(params.onErrorEventName, params.onerror);
        };

        function getParamsForImageXhr(link) {
            function onImageHeadersLoad(responseHeaders) {
                var _this = this;

                var headers = responseHeaders.split('\n');
                //console.log(response, headers);
                var contentLength = getContentLength(headers);

                if (!contentLength) {
                    onImageHeadersLoadError();
                    return;
                }

                if (contentLength - maxContentLength < 0) {
                    var imgInfoDiv = document.createElement("div");
                    var img = new Image();

                    img.style.maxHeight = maxImgHeight + "px";
                    img.onload = function (e) {
                        imgInfoDiv.appendText('(' + _this.naturalWidth + 'x' + _this.naturalHeight + ', ' + formatSize(contentLength) + ')');

                        onImageLoadEventHandler(e);
                    };
                    img.src = link.href;
                    link.innerHTML = "";
                    link.className += " imglink";

                    link.appendChild(img);
                    link.appendChild(imgInfoDiv);
                } else {
                    var tooBigView = new ImageFailView('[size is too big: ' + formatSize(contentLength) + ']');

                    link.appendText(" ");
                    tooBigView.appendTo(link);
                }
            }

            function onImageHeadersLoadError() {
                var unknownSizeView = new ImageFailView("[unknown size]");
                link.appendText(" ");
                unknownSizeView.appendTo(link);
            }

            return {
                method: 'HEAD',
                url: link.href,
                onload: onImageHeadersLoad,
                onerror: onImageHeadersLoadError
            };
        }
    };

    function formatSize(size) {
        var res = '';
        if (size < 1024) {
            res = 'b';
        } else if (size >= 1024 && size < 1024 * 1024) {
            res = 'Kb';
            size /= 1024;
        } else if (size >= 1024 * 1024) {
            res = 'Mb';
            size /= 1024 * 1024;
        }
        size = parseFloat(size.toFixed(2));
        return String.format("{0} {1}", size, res);
    }

    function getContentLength(headers) {
        for (var j = 0; j < headers.length; j++) {
            if (/^Content-Length/i.test(headers[j])) {
                return parseInt(headers[j].substring('Content-Length: '.length));
            }
        }
        return undefined;
    }

    // file: KeepScrollBottomView.js
    // done
    var KeepScrollBottomView = function KeepScrollBottomView(element) {
        var that = this;
        element.scrollToBottom();

        var threshold = parseFloat(element.attributes.getNamedItem("keep-scroll-threshold").value);

        function isScrolledToBottom() {
            return element.scrollHeight - element.offsetHeight - threshold <= element.scrollTop;
        }

        // keep track whether the element is scrolled to the bottom or not
        // (we need to know it on a resize event and after adding new posts)
        var wasScrolledToBottom = true;
        function keepTrackOfScrollStatus() {
            wasScrolledToBottom = isScrolledToBottom();
        }
        element.addEventListener("scroll", keepTrackOfScrollStatus, false);

        var resizeTimeout = null;
        function keepScrollBottomOnResize() {
            if (resizeTimeout) return;
            that.keepScrollBottom();

            resizeTimeout = setTimeout(function () {
                resizeTimeout = null;
            }, 40); // 25fps
        }
        addEventListener("resize", keepScrollBottomOnResize, false);

        that.keepScrollBottom = function () {
            if (wasScrolledToBottom) element.scrollToBottom();
        };
    };

    // file: ChatFrameView.js

    var ChatFrameView = function ChatFrameView(initialPosts) {
        var that = this;

        var chatFrameDiv = document.getElementById("chatframe");
        var keepScrollBottomView = new KeepScrollBottomView(chatFrameDiv);
        var postViews = []; // all posts that are currently displayed

        var chatPostFormView = ChatPostFormView.getInstance();

        chatPostFormView.setOnAfterAutoresizeEventHandler(keepScrollBottomView.keepScrollBottom);
        chatPostFormView.setFnOnHideChange(function () {
            return that.reloadChatFrame();
        });

        function loadChatFrame(posts) {
            enableKatex(doEnableKatex()).then(function () {
                appendPosts(posts, null);
                keepScrollBottomView.keepScrollBottom();

                doEnableEndlessScroll() ? that.enableEndlessScroll() : that.disableEndlessScroll();
            });
        }

        function highlightCodeIfNeeded() {
            if (doHighlightCode()) {
                highlightCode(chatFrameDiv);
            }
        }

        function createPostViewAndSetEventHandlers(post) {
            function onClickPostId() {
                var isAtBeginning = chatPostFormView.getTextBeforeSelectionEnd().trim().length === 0;

                if (post.thread.board.name === config.chat.boardName || !isAtBeginning) {
                    chatPostFormView.insertPostId(post.postId);
                } else {
                    var boardName = post.thread.board.name;
                    var threadId = toSimpleUuid(post.thread.threadId);
                    var postId = PostBodyView.getPostIdText(post.postId);

                    chatPostFormView.insertText('/' + boardName + '/' + threadId + ' ' + postId);
                }
            }

            var postView = new ChatPostView(post);
            postView.setOnClickPostIdEventHandler(onClickPostId);
            return postView;
        }

        function appendPost(post, previousPost) {
            appendDaysSeparatorIfNeeded(previousPost, post);

            var postView = createPostViewAndSetEventHandlers(post);
            postView.appendTo(chatFrameDiv);
            return postView;
        }

        function prependPost(post, previousPost, element) {
            prependDaysSeparatorIfNeeded(previousPost, post, element);

            var postView = createPostViewAndSetEventHandlers(post);
            postView.insertBefore(element);
            return postView;
        }

        function appendPosts(posts, lastPost) {
            ChatFrameView.appendPosts(posts, lastPost, appendPost, postViews, showImages);
            highlightCodeIfNeeded();
        }

        function prependPosts(posts, firstPost) {
            var prevHeight = chatFrameDiv.scrollHeight;
            var firstElement = chatFrameDiv.firstChild;

            var newPostViews = [];
            var lastAddedPost = reducePosts(posts, function (previousPost, post) {
                var postView = prependPost(post, previousPost, firstElement);
                newPostViews.push(postView);
            }, null);

            if (newPostViews.length < 1) return;

            prependDaysSeparatorIfNeeded(lastAddedPost, firstPost, firstElement);
            postViews = newPostViews.concat(postViews);

            var height = chatFrameDiv.scrollHeight;

            if (chatFrameDiv.scrollTop < 50) {
                chatFrameDiv.scrollTop += height - prevHeight;
            }

            if (doShowImgs()) showImages();
            highlightCodeIfNeeded();
        }

        function getLastPost() {
            var lastPostView = postViews.last();
            if (!lastPostView) return null;
            return lastPostView.getPost();
        }

        function getFirstPost() {
            var firstPostView = postViews[0];
            if (!firstPostView) return null;
            return firstPostView.getPost();
        }

        var prevScrollTop = Infinity;

        function endlessScrollHandler() {
            var threshold = 40;

            if (chatFrameDiv.scrollTop < threshold && chatFrameDiv.scrollTop < prevScrollTop && prevScrollTop >= threshold || chatFrameDiv.scrollTop === 0 && chatFrameDiv.scrollHeight === chatFrameDiv.offsetHeight) {

                var firstPost = getFirstPost();
                if (firstPost) {
                    var postsPromise = getPostsBefore$1(firstPost.postId);

                    postsPromise.then(function (newPosts) {
                        return prependPosts(newPosts, firstPost);
                    });
                }
            }
            prevScrollTop = chatFrameDiv.scrollTop;
        }

        that.enableEndlessScroll = function () {
            prevScrollTop = Infinity;
            chatFrameDiv.addEventListener("scroll", endlessScrollHandler, false);
        };
        that.disableEndlessScroll = function () {
            chatFrameDiv.removeEventListener("scroll", endlessScrollHandler, false);
        };

        function prependDaysSeparator(date, element) {
            var daysSeparatorView = new DaysSeparatorView(date);
            daysSeparatorView.insertBefore(element);
        }

        function appendDaysSeparatorIfNeeded(previousPost, post) {
            ChatFrameView.appendDaysSeparatorIfNeeded(previousPost, post, chatFrameDiv);
        }

        function prependDaysSeparatorIfNeeded(previousPost, post, element) {
            if (ChatFrameView.isDaysSeparatorNeeded(previousPost, post)) {
                prependDaysSeparator(post.postDate, element);
            }
        }

        function reducePosts(posts, reducer, someEdgePost) {
            ChatFrameView.reducePosts(posts, reducer, someEdgePost);
        }

        function showImages() {
            ChatFrameView.showImages(chatFrameDiv, onImageLoad);
        }

        function onImageLoad(e) {
            var img = e.target;
            // console.log(img.naturalWidth, img.naturalHeight);
            chatFrameDiv.scrollTop += img.height;
            // console.log(img.height, container);
        }

        function appendNewPosts(newPosts) {
            var lastPost = getLastPost();
            appendPosts(newPosts, lastPost);
            keepScrollBottomView.keepScrollBottom();

            if (lastPost.postId !== getLastPost().postId) {
                chatPostFormView.onNewPostsReceived();
            }
        }

        // Posts awaiting to be added (it might be that some of them are already
        // displayed, we must avoid showing them twice).
        // These posts are from event source only.
        var tmpPosts = [];

        var isGapPreventionInProgress = false;

        // this fn is used only after receiving new posts from EventSource
        function handleNewPostsFromEventSource(newPosts) {
            if (isGapPreventionInProgress) {
                tmpPosts = tmpPosts.concat(newPosts);
                return;
            }
            appendNewPosts(newPosts);
        }

        that.onConnected = function () {
            var lastPost = getLastPost();
            if (!lastPost) return Promise.resolve();

            isGapPreventionInProgress = true;

            // console.log(
            //     "Gap prevention started. Starting request GetMessagesAfter/",
            //     lastPost.getId()
            // );

            var postsPromise = getPostsAfter$1(lastPost.postId);

            postsPromise.then(function (newPosts) {
                appendNewPosts(newPosts);
                // console.log("Gap prevention. Displayed new posts:", getIds(newPosts));

                // Check tmpPosts. Exclude posts from newPosts,
                // because we've already displayed them.
                var ids = newPosts.map(function (p) {
                    return p.postId;
                });
                var posts = tmpPosts.filter(function (p) {
                    return ids.indexOf(p.postId) === -1;
                });

                // display these new posts
                appendNewPosts(posts);
                // console.log("While gap prevention ES received these posts:", getIds(tmpPosts));
                // console.log("And only these were displayed:", getIds(posts));
                tmpPosts = [];

                // stop collecting posts from EventSource to tmpPosts
                // because we have prevented a possible gap
                // and now we can display posts from EventSource right away
                isGapPreventionInProgress = false;
            }, function (e) {
                // nothing to handle: EvenSource will be reconnected after some time anyway.
            });
        };

        that.onServerEventReceived = handleNewPostsFromEventSource;
        that.onServerEventError = function () {
            // todo: rethink this case
            // const lastPost = getLastPost();
            // if (!lastPost) return getEmptyPromise();
            //
            // const postsPromise = PostController.getPostsModelsAfter(lastPost.getId());
            //
            // return postsPromise.then(appendNewPosts);
        };

        function cleanChatFrame() {
            chatFrameDiv.innerHTML = "";
            postViews = [];
        }

        that.reloadChatFrame = function () {
            return Promise.all([enableKatex(doEnableKatex()), enableHljs(doHighlightCode())]).then(function () {
                cleanChatFrame();

                getRecentPosts$1().then(function (posts) {
                    return loadChatFrame(posts);
                });
            });
        };

        loadChatFrame(initialPosts);
    };

    ChatFrameView.reducePosts = function (posts, reducer, someEdgePost) {
        return posts.reduce(function (previousPost, post) {
            var boardName = post.thread.board.name;
            if (boardName && boardName !== config.chat.boardName && doShowOnlyChatPosts()) {
                return post;
            }
            if (isPostIgnored(post) || post.hiddenByOp) {
                return post;
            }
            reducer(previousPost, post);
            return post;
        }, someEdgePost);
    };

    ChatFrameView.appendPosts = function (posts, lastPost, appendPost, postViews, showImages) {
        ChatFrameView.reducePosts(posts, function (previousPost, post) {
            var postView = appendPost(post, previousPost);
            postViews.push(postView);
        }, lastPost);

        if (doShowImgs()) showImages();
    };

    ChatFrameView.showImages = function (container, onImageLoad) {
        var maxImgHeight = getMaxImgHeight();
        var maxContentLength = getMaxContentLength();

        function forEachImgLink(fn) {
            var links = container.getElementsByClassName("extlink");
            links.forEach(function (link) {
                if (/\.(?:jpg|jpeg|gif|png)$/gi.test(link.innerHTML)) {
                    fn(link);
                }
            });
        }

        forEachImgLink(function (link) {
            var imageView = new ImageView(maxContentLength, maxImgHeight);
            imageView.setOnImageLoadEventHandler(onImageLoad);
            imageView.insertImage(link);
        });
    };

    ChatFrameView.appendDaysSeparatorIfNeeded = function (previousPost, post, container) {
        if (ChatFrameView.isDaysSeparatorNeeded(previousPost, post)) {
            appendDaysSeparator(post.postDate);
        }

        function appendDaysSeparator(date) {
            var daysSeparatorView = new DaysSeparatorView(date);
            daysSeparatorView.appendTo(container);
        }
    };

    ChatFrameView.isDaysSeparatorNeeded = function (previousPost, post) {
        return previousPost && post && previousPost.postDate.withoutTime() < post.postDate.withoutTime();
    };

    // file: ShadeView.js
    // done
    var ShadeView = function () {
        var that = {};

        that.addShade = function () {
            var shade = document.createElement("div");
            shade.id = "shade";
            document.body.appendChild(shade);
        };

        that.removeShade = function () {
            var shade = document.getElementById("shade");
            shade.parentNode.removeChild(shade);
        };

        return that;
    }();

    // file: SettingsView.js

    var SettingsView = function SettingsView() {
        var that = this;

        var modalsContainerDivE = document.getElementById("modals-container");
        var settingsDivE = document.getElementById("settings");
        var settingsButtonE = document.getElementById("settingsButton");

        var saveButtonE = document.getElementById("save");
        var cancelButtonE = document.getElementById("cancel");

        var elements = getSettingsElements();

        {
            elements.ignoreCyrillicInputE.getClosestByClassName("row").hide();
        }

        that.show = function () {
            ShadeView.addShade();
            setSettingsBox();
            modalsContainerDivE.show();
            settingsDivE.show();
            return false;
        };

        that.hide = function () {
            ShadeView.removeShade();
            modalsContainerDivE.hide();
            settingsDivE.hide();
        };

        that.setFunctionToCallAfterSave = function (fn) {
            that.afterSave = fn;
        };

        function saveSettings() {
            // todo: wrap everything in a form
            var validate = (elements.clientIdInputE.reportValidity || elements.clientIdInputE.checkValidity).bind(elements.clientIdInputE);

            if (!validate()) return;

            setMaxContentLength(elements.maxImgSizeInputE.value);
            setMaxImgHeight(elements.maxImgHeightInputE.value);

            setShowImgsFlag(elements.showImgsInputE.checked);
            setEmbedBitchuteFlag(elements.embedBitchuteInputE.checked);
            setShowOnlyChatPostsFlag(elements.showOnlyChatPostsE.checked);
            setIgnoreCyrillicFlag(elements.ignoreCyrillicInputE.checked);
            setEnableEndlessScrollFlag(elements.enableEndlessScrollInputE.checked);
            setEnableKatexFlag(elements.enableKatexInputE.checked);
            setHighlightCodeFlag(elements.highlightCodeInputE.checked);
            setMaxEntropy(elements.maxEntropyInputE.value);
            setRegexpFilters(elements.filtersTextareaE.value);
            setColorsFilters(elements.colorsFiltersTextareaE.value);

            setClientId(elements.clientIdInputE.value);

            that.hide();
            if (that.afterSave) that.afterSave();
        }

        function setSettingsBox() {
            elements.maxImgSizeInputE.value = getMaxContentLength();
            elements.maxImgHeightInputE.value = getMaxImgHeight();

            elements.showImgsInputE.checked = doShowImgs();
            elements.embedBitchuteInputE.checked = doEmbedBitchute();
            elements.showOnlyChatPostsE.checked = doShowOnlyChatPosts();
            elements.ignoreCyrillicInputE.checked = doIgnoreCyrillic();
            elements.enableEndlessScrollInputE.checked = doEnableEndlessScroll();
            elements.enableKatexInputE.checked = doEnableKatex();
            elements.highlightCodeInputE.checked = doHighlightCode();
            elements.maxEntropyInputE.value = getMaxEntropy();
            elements.filtersTextareaE.value = getRegexpFilters();
            elements.colorsFiltersTextareaE.value = getColorsFilters();

            elements.clientIdInputE.disabled = true;
            getOrCreateClientId().then(function (clientId) {
                elements.clientIdInputE.disabled = false;
                elements.clientIdInputE.value = clientId;
            });
        }

        settingsButtonE.onclick = that.show;
        saveButtonE.onclick = saveSettings;
        cancelButtonE.onclick = that.hide;

        window.addEventListener("keydown", function (event) {
            if (isEscape(event) && !settingsDivE.isHidden()) {
                that.hide();
            }
        }, true);
    };

    function getSettingsElements() {
        var elements = {};

        elements.maxImgSizeInputE = document.getElementById("maxImgSize");
        elements.maxImgHeightInputE = document.getElementById("maxImgHeight");

        elements.showImgsInputE = document.getElementById("showImgsP");
        elements.embedBitchuteInputE = document.getElementById("embedBitchuteP");
        elements.showOnlyChatPostsE = document.getElementById("showOnlyChatPostsP");
        elements.ignoreCyrillicInputE = document.getElementById("ignoreCyrillicP");
        elements.enableEndlessScrollInputE = document.getElementById("enableEndlessScrollP");
        elements.enableKatexInputE = document.getElementById("enableKatexP");
        elements.highlightCodeInputE = document.getElementById("highlightCodeP");
        elements.maxEntropyInputE = document.getElementById("maxEntropyP");
        elements.filtersTextareaE = document.getElementById("filtersP");
        elements.colorsFiltersTextareaE = document.getElementById("colorsFilters");

        elements.clientIdInputE = document.getElementById("clientIdP");

        return elements;
    }

    // file: HelpView.js

    var HelpView = function HelpView() {
        var modalsContainerDiv = document.getElementById("modals-container");
        var helpDiv = document.getElementById("help");
        var helpContentE = document.getElementById('help_content');
        var helpButton = document.getElementById("helpButton");
        var closeButton = document.getElementById("close");

        function show() {
            ShadeView.addShade();
            helpDiv.show();
            modalsContainerDiv.show();
            return false;
        }

        function hide() {
            ShadeView.removeShade();
            helpDiv.hide();
            modalsContainerDiv.hide();
            return false;
        }

        helpButton.onclick = show;
        closeButton.onclick = hide;

        window.addEventListener("keydown", function (event) {
            if (isEscape(event) && !helpDiv.classList.contains("hidden")) {
                hide();
            }
        }, true);

        // const badgeE = document.createElement('span');
        // badgeE.classList.add('badge');
        // badgeE.appendText('!');
        // helpButton.appendChild(badgeE);
    };

    function _classCallCheck$4(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var ExportV = function ExportV() {
        _classCallCheck$4(this, ExportV);

        var btnE = $("#exportButton");

        btnE.addEventListener('click', function (e) {
            return exportUiState();
        }, false);
    };

    function exportUiState() {
        var linkE = document.createElement("a");
        linkE.hide();

        var uiState = {
            ui: "ui2",
            version: config.version,
            localStorage: {}
        };

        Object.keys(localStorage).map(function (key) {
            if (key === key.toLowerCase()) {
                return;
            }
            uiState.localStorage[key] = getItem$1(key);
        });

        var blob = new Blob([JSON.stringify(uiState, null, 2)], { type: 'application/json' });

        var url = URL.createObjectURL(blob);
        linkE.href = url;
        linkE.download = "pfpmd_ui2_settings.json";

        document.body.appendChild(linkE);
        linkE.click();
        linkE.remove();
        URL.revokeObjectURL(url);
    }

    function _classCallCheck$5(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

    var ImportV = function ImportV() {
        _classCallCheck$5(this, ImportV);

        var btnE = $("#importButton");

        btnE.addEventListener('click', function (e) {
            return uploadUiState();
        }, false);
    };

    function importUiState(uiState) {
        // validate uiState
        if (!uiState.ui) {
            showError("Missing key: \"ui\".");
            return;
        }
        if (uiState.ui !== "ui2") {
            showError("Wrong ui value: expected \"ui2\"; got \"" + uiState.ui + "\".");
            return;
        }
        if (!uiState.version) {
            showError("Missing key: \"version\".");
            return;
        }
        if (uiState.version.constructor !== "".constructor) {
            showError("\"version\" must be a string");
            return;
        }
        if (!/^(0|[1-9]\d*)\.(0|[1-9]\d*)\.(0|[1-9]\d*)$/.test(uiState.version)) {
            showError("Wrong version value: \"" + uiState.version + "\" doesn't match \n            \"^(0|[1-9]\\d*)\\.(0|[1-9]\\d*)\\.(0|[1-9]\\d*)$\" pattern.\"");
            return;
        }
        if (!uiState.localStorage) {
            showError("Missing key: \"localStorage\".");
            return;
        }
        if (uiState.localStorage.constructor !== {}.constructor) {
            showError("\"localStorage\" must be an object.");
            return;
        }

        Object.keys(uiState.localStorage).map(function (key) {
            return setItem$1(key, uiState.localStorage[key], undefined);
        });
        location.reload();
    }

    function uploadUiState() {
        var inputE = document.createElement("input");
        inputE.hide();
        inputE.type = "file";
        inputE.accept = "application/json";
        document.body.appendChild(inputE);

        inputE.addEventListener('change', function (e) {
            var reader = new FileReader();

            reader.onload = function (event) {
                var uiState = void 0;
                try {
                    uiState = JSON.parse(event.target.result);
                } catch (err) {
                    showError("The file must be a json file.");
                    return;
                }
                importUiState(uiState);
            };

            reader.readAsText(e.target.files[0]);
            inputE.remove();
        }, false);

        inputE.click();
    }

    function showError(message) {
        return showNotification(config.notification.type.error, message, 5000);
    }

    // file: ChatPageView.js

    var ChatPageView = function () {
        var that = {};

        that.start = function (recentPostsPromise) {
            function setTitle() {
                TitleView.set("pfpmd chat");
            }

            setTitle();

            document.body.onclick = setTitle;
            document.addEventListener("keydown", setTitle, false);

            recentPostsPromise.then(function (posts) {
                var chatFrameView = new ChatFrameView(posts);

                var chatSettingsView = new SettingsView();
                chatSettingsView.setFunctionToCallAfterSave(chatFrameView.reloadChatFrame);

                new HelpView();
                new ExportV();
                new ImportV();

                initAliveTabsService();
                subscribe();

                addConnectedEventListener(chatFrameView.onConnected);
                addPostsListener(function (_, post) {
                    return chatFrameView.onServerEventReceived([post]);
                });
                addErrorEventListener(chatFrameView.onServerEventError);
            });
        };

        return that;
    }();

    // file: ChatController.js

    function loadChat() {
        TitleView.set("pfpmd chat");
        document.body.id = "chat";
        document.getElementById("chat-menu-item").classList.add("active");
        document.getElementById("chat-body").content.appendAfter(document.getElementById("menu"));

        var recentPostsPromise = getRecentPosts$1();

        new AbstractPostFormView(publishParts);
        new ChatPostFormView();

        ChatPageView.start(recentPostsPromise);
    }

    function publishParts(parts, newPost) {
        PostSubmitter.publishParts(parts, newPost).then(function (threadId) {
            // openThreadBottom(boardName, threadId);
        });
    }

    // file: BoardInfoBaseView.js
    // done
    var BoardInfoBaseView = function () {
        var that = {};

        that.initialize = function () {
            that.boardNameInputWrapper = document.getElementById("boardNameInputWrapper");
            that.boardNameInput = document.getElementById("boardNameInput");

            that.boardDescriptionElement = document.getElementById("board-description");
            that.h1 = document.createElement("h1");
        };

        that.addBoardLink = function () {
            that.boardLink = document.createElement("a");
            that.boardLink.appendChild(that.h1);
            that.boardDescriptionElement.appendChild(that.boardLink);
        };

        that.addTextOnly = function () {
            that.boardDescriptionElement.appendChild(that.h1);
        };

        return that;
    }();

    // file: BoardInfoView.js
    // done
    var BoardInfoView = function () {
        var that = {};
        var base = BoardInfoBaseView;

        that.set = function (board, isThreadPage) {
            base.initialize();
            if (!board.name) {
                base.boardDescriptionElement.style.display = "none";
                return;
            }

            var boardInfoText = "/" + board.name + "/ " + (board.description ? "- " + board.description : "");
            base.h1.appendText(boardInfoText);

            if (isThreadPage) {
                base.addBoardLink();
                base.boardLink.href = getBoardUrl(board.name);
            } else {
                base.addTextOnly();
            }

            base.boardNameInputWrapper.style.display = "none";
            base.boardNameInput.value = board.name;
        };
        return that;
    }();

    // file: ReplyFormBaseView.js
    // done
    var ReplyFormBaseView = function ReplyFormBaseView() {
        var that = this;

        that.replyForm = document.createElement("form");
        that.replyheadDiv = document.createElement("div");

        that.boardInput = document.createElement("input");
        that.replyheadDiv.appendChild(that.boardInput);

        that.threadInput = document.createElement("input");
        that.replyheadDiv.appendChild(that.threadInput);

        that.replyButton = document.createElement("button");
        that.replyheadDiv.appendChild(that.replyButton);

        that.replyForm.appendChild(that.replyheadDiv);

        that.replybodyDiv = document.createElement("div");

        that.postTextareaBaseView = new TextboxBaseView("textarea");
        that.postTextareaBaseView.appendTo(that.replybodyDiv);

        that.replyForm.appendChild(that.replybodyDiv);

        that.appendTo = function (element) {
            element.appendChild(that.replyForm);
        };
    };

    // file: ReplyFormView.js
    // done
    var ReplyFormView = function ReplyFormView(boardName, threadId) {
        var that = this;
        var base = new ReplyFormBaseView();

        base.replyForm.className = "replyform";
        base.replyForm.action = "/post";
        base.replyForm.method = "post";

        base.replyheadDiv.className = "replyhead";

        base.boardInput.name = "board";
        base.boardInput.setAttribute("type", "hidden");
        base.boardInput.value = boardName;

        base.threadInput.name = "thread";
        base.threadInput.setAttribute("type", "hidden");
        base.threadInput.value = threadId;

        base.replyButton.className = "replybtn";
        base.replyButton.type = "submit";
        base.replyButton.title = "Ctrl+Enter";
        base.replyButton.appendText("Reply");

        base.replybodyDiv.className = "replybody";

        var postTextareaView = new TextboxView(base.postTextareaBaseView);
        postTextareaView.setName("post");
        postTextareaView.setRequired();
        postTextareaView.setMaxLength(config.maxPostLength);
        postTextareaView.setId(threadId + 'tbx');
        postTextareaView.addOnKeyDownEventHandler(ctrlEnter);
        postTextareaView.autoresize();

        var abstractPostFormView = AbstractPostFormView.getInstance();
        var basePostFormView = new abstractPostFormView.BasePostFormView(postTextareaView);
        that.insertPostId = basePostFormView.insertPostId;

        base.replyButton.onclick = reply;

        function reply() {
            if (!base.replyForm.checkValidity()) return;

            var boardName = base.boardInput.value;
            var threadId = base.threadInput.value;

            try {
                var parts = getParts();
                var onPartIsSent = basePostFormView.getOnPartIsSentFunction(parts.length);

                basePostFormView.publishParts(parts, {
                    boardName: boardName,
                    subject: null,
                    threadId: threadId,
                    onPartIsSent: onPartIsSent
                });
            } catch (err) {
                console.log(err);
                return false;
            }
            return false;
        }

        function ctrlEnter(e) {
            if (isCtrlEnter(e)) reply();
        }

        function getParts() {
            return PostBodyView.getParts(postTextareaView.getValue());
        }

        this.hide = function () {
            return base.replyForm.hide();
        };
        this.show = function () {
            return base.replyForm.show();
        };

        that.appendTo = base.appendTo;
    };

    // file: PostsSeparatorView.js
    // done
    var PostsSeparatorView = function () {
        var that = {};

        that.appendTo = function (element) {
            var separatorHr = document.createElement("hr");
            separatorHr.className = "post-separator";
            element.appendChild(separatorHr);
        };

        return that;
    }();

    // file: AbstractThreadView.js

    var AbstractThreadView = function () {
        var that = {};

        that.appendPost = function (element, post, previousPost, replyFormView, isOp) {
            var postsHaveSameColors = post && previousPost && post.bg === previousPost.bg && post.fg === previousPost.fg;
            if (postsHaveSameColors) PostsSeparatorView.appendTo(element);

            var postView = new ChanPostView(post, postsHaveSameColors, false, isOp);
            postView.appendTo(element);

            function onClickPostId() {
                replyFormView.insertPostId(post.postId);
            }

            postView.setOnClickPostIdEventHandler(onClickPostId);
        };

        return that;
    }();

    // file: ThreadInfoBaseView.js
    // done
    var ThreadInfoBaseView = function ThreadInfoBaseView() {
        var that = this;
        that.threadInfoDiv = document.createElement("div");

        that.threadSpan = document.createElement("span");
        that.threadInfoDiv.appendChild(that.threadSpan);

        that.threadLink = document.createElement("a");
        that.threadNameSpan = document.createElement("span");

        that.threadLink.appendChild(that.threadNameSpan);
        that.threadInfoDiv.appendChild(that.threadLink);

        that.addBoardPart = function () {
            that.boardSpan = document.createElement("span");
            that.threadInfoDiv.insertBefore(that.boardSpan, that.threadSpan);

            that.boardLink = document.createElement("a");
            that.boardNameSpan = document.createElement("span");

            that.boardLink.appendChild(that.boardNameSpan);
            that.threadInfoDiv.insertBefore(that.boardLink, that.threadSpan);
        };

        that.appendTo = function (element) {
            element.appendChild(that.threadInfoDiv);
        };
    };

    // file: ThreadInfoView.js
    // done
    var ThreadInfoView = function ThreadInfoView(thread, doAddBoardPart) {
        var that = this;

        that.getThreadHrefP0 = function () {
            return getThreadUrl(thread.board.name, thread.threadId, 0);
        };

        var base = new ThreadInfoBaseView();

        base.threadInfoDiv.className = "thread-info";

        if (doAddBoardPart) {
            base.addBoardPart();
            base.boardSpan.appendText("Board: ");

            base.boardLink.className = "board-id";
            base.boardLink.href = getBoardUrl(thread.board.name);

            base.boardNameSpan.appendText(" /" + thread.board.name + "/");
        }

        base.threadSpan.appendText(" " + (doAddBoardPart ? "t" : "T") + "hread: ");
        base.threadLink.className = "thread-id";
        base.threadLink.href = getThreadUrl(thread.board.name, thread.threadId);

        base.threadNameSpan.appendText(toShortcutUuid(thread.threadId) + " ``" + thread.subject + "''");

        that.appendTo = base.appendTo;
    };

    // file: NavigationBarView.js
    // done
    var NavigationBarView = function () {
        var instance = void 0;
        return function (currentPage, numberOfRows, pageSize, getUrlForPage) {
            var that = this;
            if (instance) {
                throw new Error("There cannot be more than one navigation bar");
            }
            instance = that;
            NavigationBarView.getInstance = function () {
                if (!instance) {
                    throw new Error("There is no navigation bar instance");
                }
                return instance;
            };

            var lastPageNumber = numberOfRows === 0 ? 0 : Math.floor((numberOfRows - 1) / pageSize);
            if (currentPage === null) currentPage = lastPageNumber;

            function isLastPage(page) {
                return lastPageNumber === page;
            }

            // initialize the last page link
            var lastPageLink = document.getElementById("lastpage");
            if (lastPageNumber === 0) {
                // in this case navigation is not needed
                document.getElementById("navaigation-wrapper").style.display = "none";
                return;
            } else {
                lastPageLink.appendText(lastPageNumber);
                lastPageLink.href = getUrlForPage(lastPageNumber);
            }

            // initialize "Next" button
            var nextBtn = document.getElementById("nextbtn");
            if (currentPage !== null && !isLastPage(currentPage)) {
                var nextPage = currentPage + 1;
                nextBtn.onclick = function () {
                    location.href = getUrlForPage(nextPage);
                };
            } else {
                nextBtn.style.display = "none";
                lastPageLink.parentNode.className += " currentpage";
            }

            // initialize "Previous" button and the zero page link.
            var prevBtn = document.getElementById("prevbtn");
            var zeroPageLink = document.getElementById("zeropage");
            if (currentPage !== 0) {
                var prevPage = currentPage - 1;
                prevBtn.onclick = function () {
                    location.href = getUrlForPage(prevPage);
                };
            } else {
                prevBtn.style.display = "none";
                zeroPageLink.parentNode.className += " currentpage";
            }
            document.getElementById("zeropage").href = getUrlForPage(0);

            function addCurrentPageLink() {
                addPageLink(currentPage, true);
            }

            var navigationDiv = document.getElementById("navigation");
            navigationDiv.show();

            function insertBeforeLastPageLink(element) {
                navigationDiv.insertBefore(element, lastPageLink.parentNode);
            }

            function addPageLink(page, isCurrentPage) {
                var pageSpan = document.createElement("span");
                pageSpan.className = "page " + (isCurrentPage ? "currentpage" : "");

                var pageLink = document.createElement("a");
                pageLink.className = "page_link";
                pageLink.href = getUrlForPage(page);
                pageLink.appendChild(document.createTextNode(page));

                pageSpan.appendChild(pageLink);
                // <span class="page"><a class="page_link" href="#">0</a></span>
                insertBeforeLastPageLink(pageSpan);
            }

            function addGap() {
                var gapSpan = document.createElement("span");
                gapSpan.appendText(String.fromCharCode(8230));
                insertBeforeLastPageLink(gapSpan);
            }

            var fromPage = currentPage - 3;
            var toPage = currentPage + 3;

            if (currentPage < 4) {
                toPage += 4 - currentPage;
            }
            if (currentPage > lastPageNumber - 4) {
                fromPage -= currentPage - lastPageNumber + 4;
            }

            fromPage = fromPage < 1 ? 1 : fromPage;
            toPage = toPage >= lastPageNumber ? lastPageNumber - 1 : toPage;

            if (fromPage > 1) {
                addGap();
            }
            for (var i = fromPage; i <= toPage; i++) {
                if (i === currentPage) {
                    addCurrentPageLink();
                } else {
                    addPageLink(i, false);
                }
            }
            if (toPage < lastPageNumber - 1) {
                addGap();
            }
        };
    }();

    // file: ChanTitleView.js

    var ChanTitleView = function () {
        var that = {};

        that.set = function (boardName, threadId, subject, pageNumber) {
            var pageNumberText = pageNumber > 0 ? "pfpmd - page " + pageNumber : "";
            if (!boardName) {
                TitleView.add(pageNumberText);
                return;
            }

            TitleView.set("/" + boardName + "/" + (threadId ? toShortcutUuid(threadId) : "") + (subject ? " ``" + subject + "''" : "") + pageNumberText);
        };
        return that;
    }();

    // file: BlindView.js
    // done
    var BlindView = function () {
        var that = {};

        that.hide = function () {
            var blind = document.getElementById("wheel");
            blind.className += " hvr-bounce-to-bottom-start";
            setTimeout(function () {
                blind.style.display = "none";
            }, 600);
        };

        return that;
    }();

    // file: page_number_service.js

    function getThreadPageNumber(skipped) {
        var pageSize = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : config.pageSize.general;

        // Note: page number starts form 0
        // Note: skipped starts from 1 because we always skip OP

        return getPageNumber(skipped - 1, pageSize);
    }

    function getPageNumber(skipped) {
        var pageSize = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : config.pageSize.general;

        // Note: page number starts form 0

        return skipped <= 0 ? 0 : Math.floor(skipped / pageSize);
    }

    // file: ThreadView.js

    var ThreadView = function ThreadView(threadPageResponse) {
        var threadPage = threadPageResponse.data;
        var board = threadPage.thread.board;
        var threadId = threadPage.thread.threadId;
        var pageNumber = getThreadPageNumber(threadPageResponse.skipped, config.pageSize.general);

        BoardInfoView.set(board, true);

        var threadContainer = document.getElementById("thread");

        var replyFormView = new ReplyFormView(board.name, threadId);

        function appendPost(post, previousPost, isOp) {
            AbstractThreadView.appendPost(threadContainer, post, previousPost, replyFormView, isOp);
        }

        var threadInfoView = new ThreadInfoView(threadPage.thread, false);
        threadInfoView.appendTo(threadContainer);

        if (isThreadIgnored(threadPage.thread, threadPage.op)) {
            BlindView.hide();
            return;
        }
        appendPost(threadPage.op, null, true);

        threadPage.posts.reduce(function (previousPost, post) {
            appendPost(post, previousPost);
            return post;
        }, pageNumber > 0 ? null : threadPage.op); // there is a bug here!

        function getUrlForPage(page) {
            return getThreadUrl(board.name, threadId, page);
        }

        var navigationBarView = new NavigationBarView(pageNumber, Math.max(threadPageResponse.total - 1, 0), // -1 to skip OP
        config.pageSize.general, getUrlForPage);

        ChanTitleView.set(board.name, threadId, threadPage.thread.subject, pageNumber);

        replyFormView.appendTo(threadContainer);

        BlindView.hide();
    };

    // file: ShellThreadBaseView.js
    // done
    var ShellThreadBaseView = function ShellThreadBaseView() {
        var that = this;
        that.shellThreadDiv = document.createElement("div");

        that.addRecentDiv = function () {
            that.recentDiv = document.createElement("div");
            that.threadLink = document.createElement("a");

            that.recentDiv.appendChild(that.threadLink);
            that.shellThreadDiv.appendChild(that.recentDiv);
        };

        that.appendTo = function (element) {
            element.appendChild(that.shellThreadDiv);
        };
    };

    // file: ShellThreadView.js

    var ShellThreadView = function ShellThreadView(shellThread, doAddBoardPartInThreadInfo) {
        var that = this;
        var base = new ShellThreadBaseView();

        base.shellThreadDiv.className = "shell-thread";

        var thread = shellThread.thread;
        var board = thread.board;

        var replyFormView = new ReplyFormView(board.name, thread.threadId);

        function appendPost(post, previousPost, isOp) {
            AbstractThreadView.appendPost(base.shellThreadDiv, post, previousPost, replyFormView, isOp);
        }

        var threadInfoView = new ThreadInfoView(thread, doAddBoardPartInThreadInfo);
        threadInfoView.appendTo(base.shellThreadDiv);

        appendPost(shellThread.op, null, true);

        var isRecentDivNeeded = thread.postCount > 3;
        if (isRecentDivNeeded) {
            base.addRecentDiv();
            base.recentDiv.className = "recent";

            var br = base.recentDiv.insertBrBefore(base.threadLink);
            base.recentDiv.insertTextBefore("The 3 newest replies are shown below.", br);

            base.threadLink.href = threadInfoView.getThreadHrefP0();
            base.threadLink.appendText("Read this thread from the beginning.");
        }

        shellThread.posts.reduce(function (previousPost, post) {
            appendPost(post, previousPost, false);
            return post;
        }, isRecentDivNeeded ? null : shellThread.op);

        replyFormView.appendTo(base.shellThreadDiv);

        that.appendTo = base.appendTo;
    };

    // file: WarpView.js
    // done
    var WarpView = function WarpView(warpPageResponse) {
        var warpPage = warpPageResponse.data;
        var board = warpPage.board || {};

        var isBoardDefined = warpPage.board;
        BoardInfoView.set(board, false);

        var warpContainer = document.getElementById("shell-threads");
        warpPage.shellThreads.forEach(function (shellThread) {
            shellThread.thread.board = shellThread.thread.board || board;

            var shellThreadView = new ShellThreadView(shellThread, !isBoardDefined);

            shellThreadView.appendTo(warpContainer);
        });

        function getUrlForPage(page) {
            return getWarpUrl(board.name, page);
        }

        var pageNumber = getPageNumber(warpPageResponse.skipped, config.pageSize.warp);
        var navigationBarView = new NavigationBarView(pageNumber, warpPageResponse.total, config.pageSize.warp, getUrlForPage);

        ChanTitleView.set(board.name, null, null, pageNumber);
        BlindView.hide();
    };

    // file: ChanPageView.js

    var ChanPageView = function () {
        var that = {};

        that.loadThread = function (threadPagePromise) {
            document.getElementById("newThreadLink").style.display = "none";

            return threadPagePromise.then(function (response) {
                var threadView = new ThreadView(response);

                initMenu();
            });
        };

        that.loadWarp = function (warpPagePromise, isBoardDefined) {
            document.getElementById("thread").style.display = "none";
            // if (!isBoardDefined) {
            //     document.getElementById("chanLink").style.display = "none";
            // }

            warpPagePromise.then(function (response) {
                var warpView = new WarpView(response);

                initMenu();
            });
        };

        function initMenu() {
            if (doHighlightCode()) {
                highlightCode(document.body);
            }

            new SettingsView().setFunctionToCallAfterSave(function () {
                return location.reload();
            });
            new HelpView();
            new ExportV();
            new ImportV();
        }

        return that;
    }();

    // file: NewThreadFormView.js

    var NewThreadFormView = function () {
        var instance = void 0;
        return function () {
            var that = this;
            if (instance) {
                throw new Error("There cannot be more than one 'new thread' form");
            }
            instance = that;
            NewThreadFormView.getInstance = function () {
                if (!instance) {
                    throw new Error("There is no 'new thread' form instance");
                }
                return instance;
            };

            var form = document.getElementById("createform");
            var wrapper = document.getElementById("createform-wrapper");
            var subjectInput = document.getElementById("subjectInput");
            var postText = document.getElementById("postText");

            var textboxBaseView = new TextboxBaseView(null, postText);
            var textboxView = new TextboxView(textboxBaseView);
            textboxView.addOnKeyDownEventHandler(ctrlEnter);
            textboxView.autoresize();

            var boardNameInput = document.getElementById("boardNameInput");
            var btn = document.getElementById("createNewThreadBtn");

            // by default it is hidden
            wrapper.style.display = "inherit";

            var abstractPostFormView = AbstractPostFormView.getInstance();
            var basePostFormView = new abstractPostFormView.BasePostFormView(textboxView);

            btn.onclick = createNewThread;
            btn.title = "Ctrl+Enter";

            function createNewThread() {
                if (!form.checkValidity()) return;

                var boardName = getBoardName();
                var subject = subjectInput.value;

                try {
                    var parts = getParts();
                    var onPartIsSent = basePostFormView.getOnPartIsSentFunction(parts.length);

                    basePostFormView.publishParts(parts, {
                        boardName: boardName,
                        subject: subject,
                        threadId: null,
                        onPartIsSent: onPartIsSent
                    });
                } catch (err) {
                    console.log(err);
                    return false;
                }
                return false;
            }

            function ctrlEnter(e) {
                if (isCtrlEnter(e)) createNewThread();
            }

            function getBoardName() {
                return boardNameInput.value.toLowerCase();
            }

            function getParts() {
                return PostBodyView.getParts(textboxView.getValue());
            }
        };
    }();

    // file: ChanController.js

    function loadChan() {
        TitleView.set("pfpmd chan");
        document.body.id = "chan";
        document.getElementById("chan-menu-item").classList.add("active");
        document.getElementById("chan-body").content.appendAfter(document.getElementById("menu"));

        new AbstractPostFormView(publishParts$1);

        var postId = getPostId();
        if (postId) {
            loadPostPage(postId);
            return;
        }
        var threadId = getSelectedThreadId();
        var boardName = getSelectedBoardName();
        var pageNumber = getPage();

        if (threadId) {
            var threadPagePromise = getThreadPage$1(threadId, pageNumber);

            ChanPageView.loadThread(threadPagePromise).then(activateTarget);
        } else {
            var warpPagePromise = getWarpPage$1(boardName, pageNumber).then(function (r) {
                r.data.shellThreads = r.data.shellThreads.filter(function (st) {
                    return !isThreadIgnored(st.thread, st.op);
                }).map(function (st) {
                    st.posts = st.posts.filter(function (p) {
                        return !isPostIgnored(p);
                    });
                    return st;
                });
                return r;
            });
            var isBoardDefined = !!boardName;
            ChanPageView.loadWarp(warpPagePromise, isBoardDefined);
            new NewThreadFormView();
        }
        // chan.removeQuotePreviewsOnClick();
    }

    function loadPostPage(postId) {
        var threadPagePromise = getPost$1(postId, true).then(function (post) {
            var pageNumber = getThreadPageNumber(post.index);

            window.history.replaceState('', '', getThreadUrl(post.thread.board.name, post.thread.threadId, pageNumber, post.postId));

            return getThreadPage$1(post.thread.threadId, pageNumber).then(function (r) {
                r.data.posts = r.data.posts.filter(function (p) {
                    return !isPostIgnored(p);
                });
                return r;
            });
        });

        ChanPageView.loadThread(threadPagePromise).then(activateTarget);
    }

    function publishParts$1(parts, params) {
        PostSubmitter.publishParts(parts, params).then(function (newThreadId) {
            openThreadBottom(params.boardName, newThreadId);
        });
    }

    function openThreadBottom(boardName, threadId) {
        var url = getThreadUrl(boardName, threadId);

        if (location.pathname + location.search === url) {
            location.hash = "bottom";
            location.reload();
        } else {
            location.href = url + '#bottom';
        }
    }

    // file: LogsPostBaseView.js
    // done
    var LogsPostBaseView = function LogsPostBaseView() {
        var that = this;

        var chatPostBaseView = new ChatPostBaseView();

        that.postDivE = chatPostBaseView.postDivE;
        that.postLeftE = chatPostBaseView.postLeftE;
        that.infoIdE = chatPostBaseView.infoIdE;
        that.infoNameE = chatPostBaseView.infoNameE;
        that.nicknameE = chatPostBaseView.nicknameE;

        that.postIdSpanE = chatPostBaseView.postIdSpanE;
        that.postbodyDivE = chatPostBaseView.postbodyDivE;

        that.dateSpan = document.createElement("span");
        that.dateDivE = document.createElement("div");
        that.dateDivE.classList.add('tm-wrapper');
        that.dateDivE.appendChild(that.dateSpan);
        that.postDivE.insertBefore(that.dateDivE, that.postLeftE);

        that.postInfoBaseView = chatPostBaseView.postInfoBaseView;

        that.appendTo = chatPostBaseView.appendTo;
    };

    // file: LogsPostView.js

    var LogsPostView = function LogsPostView(post) {
        var that = this;
        var base = new LogsPostBaseView();

        var chatPostView = new ChatPostView(post, base);

        base.dateSpan.className = "tm";
        base.dateSpan.appendText(formatTime(post.postDate) + ' ');

        that.getPost = chatPostView.getPost;
        that.appendTo = chatPostView.appendTo;
    };

    // file: LogsFrameView.js
    // done
    var LogsFrameView = function LogsFrameView(initialPosts) {

        var logsFrameDiv = document.getElementById("chatframe");

        function loadLogsFrame(posts) {
            appendPosts(posts);

            if (doHighlightCode()) {
                highlightCode(logsFrameDiv);
            }
        }

        function appendPosts(posts) {
            ChatFrameView.appendPosts(posts, null, appendPost, [], showImages);
        }

        function appendPost(post, nextPost) {
            appendDaysSeparatorIfNeeded(post, nextPost);

            var postView = new LogsPostView(post);
            postView.appendTo(logsFrameDiv);

            return postView;
        }

        function showImages() {
            ChatFrameView.showImages(logsFrameDiv, function (x) {
                return x;
            });
        }

        function appendDaysSeparatorIfNeeded(previousPost, post) {
            ChatFrameView.appendDaysSeparatorIfNeeded(previousPost, post, logsFrameDiv);
        }

        loadLogsFrame(initialPosts);
    };

    // file: LogsPageView.js

    var LogsPageView = function () {
        var that = {};

        that.load = function (logsPagePromise, pageNumber) {
            logsPagePromise.then(function (logsPageResponse) {
                var posts = logsPageResponse.data;

                var logsFrameView = new LogsFrameView(posts);

                function getUrlForPage(page) {
                    return getLogsUrl(page);
                }

                var navigationBarView = new NavigationBarView(pageNumber, logsPageResponse.total, config.pageSize.general, getUrlForPage);

                new SettingsView().setFunctionToCallAfterSave(function () {
                    return location.reload();
                });
                new HelpView();
                new ExportV();
                new ImportV();
            });
        };

        return that;
    }();

    // file: LogsController.js

    function loadLogs() {
        TitleView.set("pfpmd chat logs");
        document.body.id = "logsb";
        document.getElementById("logs-menu-item").classList.add("active");
        document.getElementById("logs-body").content.appendAfter(document.getElementById("menu"));

        var pageNumber = getPage() || 0;

        var logsPagePromise = getLogsPage$1(pageNumber);
        // const logsPagePromise = Promise.all([
        //     repo.getLogsPage(pageNumber),
        //     repo.getLogsPage(pageNumber + 1),
        // ]).then(([response1, response2]) => {
        //
        //     const posts = response1.data.concat(response2.data);
        //     const skipped = response1.skipped + response2.skipped;
        //     const total = response2.total; // doesn't matter which, they're the same
        //
        //     return new SuccessResponse(posts, skipped, total);
        // });

        LogsPageView.load(logsPagePromise, pageNumber);
    }

    // file: main.js

    document.addEventListener("DOMContentLoaded", function () {
        // todo: call just main when tor browser bug is fixed
        initLocalStorageService(main);
    });

    function main() {
        //lsService.addStorageEventListener(); // todo: uncomment when tor browser bug is fixed

        Promise.all([enableKatex(doEnableKatex()), enableHljs(doHighlightCode())]).then(function () {
            var path = getPathName();
            if (!path.endsWith('/')) path += '/';

            if (path === '/') {
                loadChat();
            } else if (path === '/logs/') {
                loadLogs();
            } else {
                loadChan();
            }
        });
    }

}());
