// file: ChatFrameView.js
import {ChatPostFormView} from './ChatPostFormView.js'
import {ChatPostView} from './ChatPostView.js'
import {DaysSeparatorView} from './DaysSeparatorView.js'
import {ImageView} from './ImageView.js'
import {KeepScrollBottomView} from './KeepScrollBottomView.js'
import {enableKatex} from "../extra_services/katex_service";
import * as settings from "../services/settings_service";
import {config} from "../config";
import {enableHljs, highlightCode} from "../extra_services/hljs_service";
import * as repo from "../repository";
import {toSimpleUuid} from "../services/uuid_service";
import {PostBodyView} from '../Views/PostBodyView.js';

const ChatFrameView = function (initialPosts) {
    const that = this;

    const chatFrameDiv = document.getElementById("chatframe");
    const keepScrollBottomView = new KeepScrollBottomView(chatFrameDiv);
    let postViews = []; // all posts that are currently displayed

    const chatPostFormView = ChatPostFormView.getInstance();

    chatPostFormView.setOnAfterAutoresizeEventHandler(
        keepScrollBottomView.keepScrollBottom
    );
    chatPostFormView.setFnOnHideChange(() => that.reloadChatFrame());

    function loadChatFrame(posts) {
        enableKatex(settings.doEnableKatex()).then(() => {
            appendPosts(posts, null);
            keepScrollBottomView.keepScrollBottom();

            settings.doEnableEndlessScroll()
                ? that.enableEndlessScroll()
                : that.disableEndlessScroll();
        });
    }

    function highlightCodeIfNeeded() {
        if (settings.doHighlightCode()) {
            highlightCode(chatFrameDiv);
        }
    }

    function createPostViewAndSetEventHandlers(post) {
        function onClickPostId() {
            const isAtBeginning = chatPostFormView.getTextBeforeSelectionEnd().trim().length === 0;

            if (post.thread.board.name === config.chat.boardName || !isAtBeginning) {
                chatPostFormView.insertPostId(post.postId);
            } else {
                const boardName = post.thread.board.name;
                const threadId = toSimpleUuid(post.thread.threadId);
                const postId = PostBodyView.getPostIdText(post.postId);

                chatPostFormView.insertText(
                    `/${boardName}/${threadId} ${postId}`
                );
            }
        }

        const postView = new ChatPostView(post);
        postView.setOnClickPostIdEventHandler(onClickPostId);
        return postView;
    }

    function appendPost(post, previousPost) {
        appendDaysSeparatorIfNeeded(previousPost, post);

        const postView = createPostViewAndSetEventHandlers(post);
        postView.appendTo(chatFrameDiv);
        return postView;
    }

    function prependPost(post, previousPost, element) {
        prependDaysSeparatorIfNeeded(previousPost, post, element);

        const postView = createPostViewAndSetEventHandlers(post);
        postView.insertBefore(element);
        return postView;
    }

    function appendPosts(posts, lastPost) {
        ChatFrameView.appendPosts(posts, lastPost, appendPost, postViews, showImages);
        highlightCodeIfNeeded();
    }

    function prependPosts(posts, firstPost) {
        const prevHeight = chatFrameDiv.scrollHeight;
        const firstElement = chatFrameDiv.firstChild;

        const newPostViews = [];
        const lastAddedPost = reducePosts(posts, (previousPost, post) => {
            const postView = prependPost(post, previousPost, firstElement);
            newPostViews.push(postView);
        }, null);

        if (newPostViews.length < 1) return;

        prependDaysSeparatorIfNeeded(lastAddedPost, firstPost, firstElement);
        postViews = newPostViews.concat(postViews);

        const height = chatFrameDiv.scrollHeight;

        if (chatFrameDiv.scrollTop < 50) {
            chatFrameDiv.scrollTop += (height - prevHeight);
        }

        if (settings.doShowImgs()) showImages();
        highlightCodeIfNeeded();
    }

    function getLastPost() {
        const lastPostView = postViews.last();
        if (!lastPostView) return null;
        return lastPostView.getPost();
    }

    function getFirstPost() {
        const firstPostView = postViews[0];
        if (!firstPostView) return null;
        return firstPostView.getPost();
    }

    let prevScrollTop = Infinity;

    function endlessScrollHandler() {
        const threshold = 40;

        if ((chatFrameDiv.scrollTop < threshold &&
                chatFrameDiv.scrollTop < prevScrollTop &&
                prevScrollTop >= threshold)
            || (chatFrameDiv.scrollTop === 0 &&
                chatFrameDiv.scrollHeight === chatFrameDiv.offsetHeight)) {

            const firstPost = getFirstPost();
            if (firstPost) {
                const postsPromise = repo.getPostsBefore(firstPost.postId);

                postsPromise.then(newPosts => prependPosts(newPosts, firstPost));
            }
        }
        prevScrollTop = chatFrameDiv.scrollTop;
    }

    that.enableEndlessScroll = () => {
        prevScrollTop = Infinity;
        chatFrameDiv.addEventListener("scroll", endlessScrollHandler, false);
    };
    that.disableEndlessScroll = () => {
        chatFrameDiv.removeEventListener("scroll", endlessScrollHandler, false);
    };

    function prependDaysSeparator(date, element) {
        const daysSeparatorView = new DaysSeparatorView(date);
        daysSeparatorView.insertBefore(element);
    }

    function appendDaysSeparatorIfNeeded(previousPost, post) {
        ChatFrameView.appendDaysSeparatorIfNeeded(
            previousPost, post, chatFrameDiv
        );
    }

    function prependDaysSeparatorIfNeeded(previousPost, post, element) {
        if (ChatFrameView.isDaysSeparatorNeeded(previousPost, post)) {
            prependDaysSeparator(post.postDate, element);
        }
    }

    function reducePosts(posts, reducer, someEdgePost) {
        ChatFrameView.reducePosts(posts, reducer, someEdgePost);
    }

    function showImages() {
        ChatFrameView.showImages(chatFrameDiv, onImageLoad);
    }

    function onImageLoad(e) {
        const img = e.target;
        // console.log(img.naturalWidth, img.naturalHeight);
        chatFrameDiv.scrollTop += img.height;
        // console.log(img.height, container);
    }

    function appendNewPosts(newPosts) {
        const lastPost = getLastPost();
        appendPosts(newPosts, lastPost);
        keepScrollBottomView.keepScrollBottom();

        if (lastPost.postId !== getLastPost().postId) {
            chatPostFormView.onNewPostsReceived();
        }
    }

    // Posts awaiting to be added (it might be that some of them are already
    // displayed, we must avoid showing them twice).
    // These posts are from event source only.
    let tmpPosts = [];

    let isGapPreventionInProgress = false;

    // this fn is used only after receiving new posts from EventSource
    function handleNewPostsFromEventSource(newPosts) {
        if (isGapPreventionInProgress) {
            tmpPosts = tmpPosts.concat(newPosts);
            return;
        }
        appendNewPosts(newPosts);
    }

    // fn for debug only
    function getIds(posts) {
        return posts.map(p => p.postId);
    }

    that.onConnected = () => {
        const lastPost = getLastPost();
        if (!lastPost) return Promise.resolve();

        isGapPreventionInProgress = true;

        // console.log(
        //     "Gap prevention started. Starting request GetMessagesAfter/",
        //     lastPost.getId()
        // );

        const postsPromise = repo.getPostsAfter(lastPost.postId);

        postsPromise.then(newPosts => {
            appendNewPosts(newPosts);
            // console.log("Gap prevention. Displayed new posts:", getIds(newPosts));

            // Check tmpPosts. Exclude posts from newPosts,
            // because we've already displayed them.
            const ids = newPosts.map(p => p.postId);
            const posts = tmpPosts.filter(p => ids.indexOf(p.postId) === -1);

            // display these new posts
            appendNewPosts(posts);
            // console.log("While gap prevention ES received these posts:", getIds(tmpPosts));
            // console.log("And only these were displayed:", getIds(posts));
            tmpPosts = [];

            // stop collecting posts from EventSource to tmpPosts
            // because we have prevented a possible gap
            // and now we can display posts from EventSource right away
            isGapPreventionInProgress = false;
        }, (e) => {
            // nothing to handle: EvenSource will be reconnected after some time anyway.
        });
    };

    that.onServerEventReceived = handleNewPostsFromEventSource;
    that.onServerEventError = () => {
        // todo: rethink this case
        // const lastPost = getLastPost();
        // if (!lastPost) return getEmptyPromise();
        //
        // const postsPromise = PostController.getPostsModelsAfter(lastPost.getId());
        //
        // return postsPromise.then(appendNewPosts);
    };

    function cleanChatFrame() {
        chatFrameDiv.innerHTML = "";
        postViews = [];
    }

    that.reloadChatFrame = () => Promise.all([
        enableKatex(settings.doEnableKatex()),
        enableHljs(settings.doHighlightCode()),
    ]).then(() => {
        cleanChatFrame();

        repo.getRecentPosts().then(posts => loadChatFrame(posts));
    });

    loadChatFrame(initialPosts);
};

ChatFrameView.reducePosts = (posts, reducer, someEdgePost) => {
    return posts.reduce((previousPost, post) => {
        const boardName = post.thread.board.name;
        if (boardName && boardName !== config.chat.boardName
            && settings.doShowOnlyChatPosts()
        ) {
            return post;
        }
        if (settings.isPostIgnored(post) || post.hiddenByOp) {
            return post;
        }
        reducer(previousPost, post);
        return post;
    }, someEdgePost);
};

ChatFrameView.appendPosts = (posts, lastPost, appendPost, postViews, showImages) => {
    ChatFrameView.reducePosts(posts, (previousPost, post) => {
        const postView = appendPost(post, previousPost);
        postViews.push(postView);
    }, lastPost);

    if (settings.doShowImgs()) showImages();
}
;

ChatFrameView.showImages = (container, onImageLoad) => {
    const maxImgHeight = settings.getMaxImgHeight();
    const maxContentLength = settings.getMaxContentLength();

    function forEachImgLink(fn) {
        const links = container.getElementsByClassName("extlink");
        links.forEach(link => {
            if (/\.(?:jpg|jpeg|gif|png)$/gi.test(link.innerHTML)) {
                fn(link);
            }
        });
    }

    forEachImgLink(link => {
        const imageView = new ImageView(maxContentLength, maxImgHeight);
        imageView.setOnImageLoadEventHandler(onImageLoad);
        imageView.insertImage(link);
    });
};

ChatFrameView.appendDaysSeparatorIfNeeded = (previousPost, post, container) => {
    if (ChatFrameView.isDaysSeparatorNeeded(previousPost, post)) {
        appendDaysSeparator(post.postDate);
    }

    function appendDaysSeparator(date) {
        const daysSeparatorView = new DaysSeparatorView(date);
        daysSeparatorView.appendTo(container);
    }
};

ChatFrameView.isDaysSeparatorNeeded = (previousPost, post) => {
    return previousPost && post && previousPost.postDate.withoutTime() < post.postDate.withoutTime();
};

export {ChatFrameView}
