// file: ChatPostFormView.js
import {TitleView} from './TitleView.js'
import {TypingLoggerView} from './TypingLoggerView.js'
import {TextboxView} from './TextboxView.js'
import {TextboxBaseView} from '../BaseViews/TextboxBaseView.js'
import {AbstractPostFormView} from './AbstractPostFormView.js'
import {PostBodyView} from './PostBodyView.js'
import {Board} from "../models";
import * as repo from "../repository";
import {updateClientId} from "../services/client_service";
import {config} from "../config";
import {isAltEnter, isEnter, isShiftEnter} from "../services/shortcut_service";
import {parseUuid} from "../services/text_parsing_service";
import {uuidPattern} from "../services/uuid_service";
import {getColorsFilters, setColorsFilters} from "../services/settings_service";
import {showNotification} from "./notifications/notification_v";
import {parseColorsFilters} from "../services/filter_service";
// done
let instance;
const ChatPostFormView = function () {
    const that = this;
    if (instance) {
        throw new Error("There cannot be more than one 'chat post' form");
    }
    instance = that;
    ChatPostFormView.getInstance = () => {
        if (!instance) {
            throw new Error("There is no 'chat post' form instance");
        }
        return instance;
    };

    const chatPostForm = document.getElementById("chat-post-form");
    const postTextbox = document.getElementById("msgtxt");

    const postTextboxBaseView = new TextboxBaseView(null, postTextbox);
    const postTextboxView = new TextboxView(postTextboxBaseView);

    const abstractPostFormView = AbstractPostFormView.getInstance();
    const basePostFormView = new abstractPostFormView.BasePostFormView(
        postTextboxView
    );

    that.insertPostId = basePostFormView.insertPostId;
    that.insertText = basePostFormView.insertText;
    that.getTextBeforeSelectionEnd = basePostFormView.getTextBeforeSelectionEnd;

    const hideColorRe = new RegExp(`^\\/hide\\s*@${uuidPattern}`);
    const unhideColorRe = new RegExp(`^\\/unhide\\s*@${uuidPattern}`);

    function onSubmit() {
        try {
            const text = postTextboxView.getValue();
            if (!text) return;

            if (text === "/change-colors" || text === "/cc") {
                updateClientId();
                postTextboxView.clear();
                postTextboxView.setPlaceholder("Your colors were changed.");
                return;
            }
            if (text === "/help") {
                document.getElementById("helpButton").click();
                postTextboxView.clear();
                return;
            }
            if (text === "/settings") {
                document.getElementById("settingsButton").click();
                postTextboxView.clear();
                return;
            }
            const hideColorMatch = hideColorRe.exec(text);
            if (hideColorMatch) {
                const postId = hideColorMatch[1];
                hideColor(postId);
                postTextboxView.clear();
                return;
            }

            const unhideColorMatch = unhideColorRe.exec(text);
            if (unhideColorMatch) {
                const postId = unhideColorMatch[1];
                unhideColor(postId);
                postTextboxView.clear();
                return;
            }

            const boardMatch = /^\/([a-z]{1,3}) /.exec(text);
            if (boardMatch) {
                postTextboxView.clear();
                const boardName = boardMatch[1];
                const boardDescription = text.substr(boardName.length + 2);

                const board = new Board(boardName, boardDescription);

                repo.updateBoardDescription(board).then(response => {
                    // console.log(response);
                });
            } else {
                publish();
            }
        }
        catch (err) {
            console.log(err);
            return false;
        }
        return false;
    }

    let didIPost = false;

    function publish() {
        let boardName = config.chat.boardName;
        let threadId = config.chat.threadId;
        let subject = null;

        const text = postTextboxView.getValue();
        let postText = text;

        let reObj = /^\/([a-z]{1,3})\/([\-0-9a-fA-F]+) /.exec(text);
        if (reObj) {
            threadId = parseUuid(reObj[2]);

            if (threadId) {
                threadId = threadId.match;
                boardName = reObj[1];
                postText = text.substr(`/${boardName}/${threadId} `.length);
            }
        }

        reObj = /^\/([a-z]{1,3})\[(.+)\] /.exec(text);
        if (reObj) {
            threadId = null;
            boardName = reObj[1];
            subject = reObj[2];

            postText = text.substr(
                `/${boardName}[${subject}] `.length
            );
        }

        let parts = PostBodyView.getParts(postText);
        postTextboxView.clear();
        if (parts.length === 1 && parts[0] === "") return false;

        const onPartIsSent = basePostFormView.getOnPartIsSentFunction(parts.length);

        basePostFormView.publishParts(parts, {
            boardName: boardName,
            subject: subject,
            threadId: threadId,
            onPartIsSent: onPartIsSent,
        });
        didIPost = true;

        return false;
    }

    function onKeyDown(e) {
        if (isAltEnter(e)) {
            postTextboxView.insertText("\n");
        }
        else if (isEnter(e) && !isShiftEnter(e)) {
            e.preventDefault();
            onSubmit();
        }
    }

    that.onNewPostsReceived = () => {
        if (didIPost) {
            didIPost = false;
            return;
        }
        TitleView.set("[new posts] - pfpmd chat");
    };

    function runTypingLogger() {
        TypingLoggerView.runTypingLogger(postTextboxView);
    }

    chatPostForm.onsubmit = onSubmit;

    postTextboxView.addOnKeyDownEventHandler(onKeyDown);
    postTextboxView.autoresize();

    that.setOnAfterAutoresizeEventHandler = (eventHandler) => {
        postTextboxView.setOnAfterAutoresizeEventHandler(eventHandler);
    };

    let onHideChange = () => {};
    that.setFnOnHideChange = (fn) => {
        onHideChange = fn;
    };

    function hideColor(postId) {
        repo.getPost(postId).then(post => {
            const filters = parseColorsFilters(getColorsFilters());

            if (!filters.some(({bg, fg}) => post.bg === bg && post.fg === fg)) {
                filters.push({bg: post.bg, fg: post.fg});
            }

            setColorsFilters(filters.map(({bg, fg}) => `${bg}:${fg}\n`).join(''));
            onHideChange();
        }).catch(errorResponse => {
            showNotification(config.notification.type.error, errorResponse.message);
        });
    }

    function unhideColor(postId) {
        repo.getPost(postId).then(post => {
            const filters = parseColorsFilters(getColorsFilters());

            setColorsFilters(
                filters.filter(({bg, fg}) => !(bg === post.bg && fg === post.fg))
                    .map(({bg, fg}) => `${bg}:${fg}\n`).join('')
            );
            onHideChange();
        }).catch(errorResponse => {
            showNotification(config.notification.type.error, errorResponse.message);
        });
    }

    runTypingLogger();
};

export {ChatPostFormView}
