// file: PostBodyView.js
import {LinkView} from './LinkView.js'
import {PostReferenceView} from './PostReferenceView.js'
import {config} from "../config";
import {renderKatex} from "../extra_services/katex_service";
import {parsePostReference} from "../services/text_parsing_service";
import {toSimpleUuid} from "../services/uuid_service";

const PostBodyView = ((insertQuotes, splitLongText) => {
    const that = {};
    const urlRe = /^(?:(?:(?:https?|ftp):\/\/)(?:\S+(?::\S*)?@)?(?:(?!10(?:\.\d{1,3}){3})(?!127(?:\.\d{1,3}){3})(?!169\.254(?:\.\d{1,3}){2})(?!192\.168(?:\.\d{1,3}){2})(?!172\.(?:1[6-9]|2\d|3[0-1])(?:\.\d{1,3}){2})(?:[1-9]\d?|1\d\d|2[01]\d|22[0-3])(?:\.(?:1?\d{1,2}|2[0-4]\d|25[0-5])){2}(?:\.(?:[1-9]\d?|1\d\d|2[0-4]\d|25[0-4]))|(?:(?:[a-z\u00a1-\uffff0-9]+-?)*[a-z\u00a1-\uffff0-9]+)(?:\.(?:[a-z\u00a1-\uffff0-9]+-?)*[a-z\u00a1-\uffff0-9]+)*(?:\.(?:[a-z\u00a1-\uffff0-9]{2,})))(?::\d{2,5})?(?:\/[^\s]*)?|magnet:\?(?:[^\s\]\)]+))/;

    that.getFormattedText = function (text) {
        function escapeHtml(unsafe) {
            return unsafe
                .replace(/&/g, "&amp;")
                .replace(/</g, "&lt;")
                .replace(/>/g, "&gt;")
                .replace(/"/g, "&quot;")
                .replace(/'/g, "&#039;");
        }

        text = escapeHtml(text);

        text = text.replace(/\n{3,}/g, "\n\n");

        let formattedText = insertQuotes(text);
        formattedText = formatText(formattedText);

        return formattedText;
    };

    // it is used for the forms
    that.getParts = (text) => splitLongText(text);

    that.getPostIdText = (postId) => {
        return `@${toSimpleUuid(postId)} `;
    };

    function formatText(text) {
        for (let i = 0; i < text.length; i++) {
            const textToParse = text.slice(i);
            const markupFreePrefix = i > 0 ? text.substring(0, i) : "";

            const newline = parserNewline(textToParse);
            if (newline) {
                return `${markupFreePrefix}<br>${formatText(newline.rest)}`;
            }
            const bold = parseBold(textToParse);
            if (bold) {
                return `${markupFreePrefix}<b>${formatText(bold.match)}</b>${formatText(bold.rest)}`;
            }
            const italic = parseItalic(textToParse);
            if (italic) {
                return `${markupFreePrefix}<i>${formatText(italic.match)}</i>${formatText(italic.rest)}`;
            }
            const code = parseCode(textToParse);
            if (code) {
                return `${markupFreePrefix}<pre><code class='${code.lang}'>${code.match}</code></pre>${formatText(code.rest)}`;
            }
            const mono = parseMono(textToParse);
            if (mono) {
                return `${markupFreePrefix}<samp>${formatText(mono.match)}</samp>${formatText(mono.rest)}`;
            }
            const strike = parseStrike(textToParse);
            if (strike) {
                return `${markupFreePrefix}<s>${formatText(strike.match)}</s>${formatText(strike.rest)}`;
            }
            const tex = parseTeX(textToParse);
            if (tex) {
                const katexElement = document.createElement("div");
                renderKatex(tex.match, katexElement);

                return (markupFreePrefix + katexElement.innerHTML + formatText(tex.rest));
            }
            const spoiler = parseSpoiler(textToParse);
            if (spoiler) {
                return `${markupFreePrefix}<span class="spoiler">${formatText(spoiler.match)}</span>${formatText(spoiler.rest)}`;
            }
            const namedLink = parseNamedLink(textToParse);
            if (namedLink) {
                return (markupFreePrefix +
                    LinkView.formatLink(namedLink.match, namedLink.name) +
                    formatText(namedLink.rest));
            }
            const link = parseLink(textToParse);
            if (link) {
                return (markupFreePrefix +
                    LinkView.formatLink(link.match) +
                    formatText(link.rest));
            }
            const postReference = parsePostReference(textToParse);
            if (postReference) {
                const postReferenceView = new PostReferenceView();
                postReferenceView.create(postReference.match);

                return (markupFreePrefix +
                    postReferenceView.getOuterHtml() +
                    formatText(postReference.rest));
            }
        }
        return text;
    }

    function parseNamedLink(text) {
        const match = /^\[([^\n\r\]]+)\]\(([^\)]+)\)/.exec(text);
        if (!match) return null;
        const name = match[1];
        // const urlMatch = urlRe.exec(match[2]);
        // if (!urlMatch) return null;
        // const url = urlMatch[0];
        const url = match[2];
        const lengthToSkip = match[0].length;
        return {match: url, name: name, rest: text.slice(lengthToSkip)};
    }

    function parseLink(text) {
        const match = urlRe.exec(text);
        if (!match) return null;
        const url = match[0];
        return {match: url, rest: text.slice(url.length)};
    }

    function parserNewline(text) {
        if (text[0] === '\n') {
            return {rest: text.slice(1)};
        }
        return null;
    }

    function parseBold(text) {
        return parseSimpleMarkup("**", text);
    }

    function parseItalic(text) {
        return parseSimpleMarkup("//", text);
    }

    function parseMono(text) {
        return parseSimpleMarkup("`", text);
    }

    function parseTeX(text) {
        return parseSimpleMarkup("$", text);
    }

    function parseStrike(text) {
        return parseSimpleMarkup("~~", text);
    }

    function parseSpoiler(text) {
        return parseSimpleMarkup("%%", text);
    }

    function parseCode(text) {
        const code = parseSimpleMarkup("```", text);
        if (!code) return code;
        const langMatch = /^[^\s]*/.exec(code.match);
        if (!langMatch) return code;
        code.lang = langMatch[0];
        code.match = code.match.slice(code.lang.length).replace(/^\s/, "");
        return code;
    }

    function parseSimpleMarkup(markup, text) {
        if (!text.startsWith(markup)) return null;
        const idx = text.indexOf(markup, markup.length);
        if (idx < 0) return null;
        const match = text.substring(markup.length, idx);
        return {match: match, rest: text.slice(idx + markup.length)};
    }

    return that;
})(
    function insertQuotes(text, level) {
        level = level || 0;

        const hiddenQuoteSign = (() => {
            const hiddenQuoteSignSpan = document.createElement("span");
            hiddenQuoteSignSpan.className = "hidden-quote-sign";
            hiddenQuoteSignSpan.appendText(">");
            return hiddenQuoteSignSpan.outerHTML;
        })();

        return text.replace(/^(?:@[0-9a-f]+)?((?:\s*&gt;.+\n?)+)/gm, (quote) => {
            let postReference = "";
            quote = quote.replace(/^(@[0-9a-f]+)?\s*(&gt;\s*)/gm, (_, p1) => {
                postReference = p1 || "";
                return "";
            });

            quote = (level > 10 ? quote : insertQuotes(quote, level + 1));

            const blockquote = document.createElement("blockquote");
            blockquote.innerHTML = quote + " "; // hotfix. todo: refactor

            return postReference + blockquote.outerHTML;
        });
    },
    function splitLongText(text) {
        const partLength = config.maxPostLength;
        const words = text.split(/(?=\s)/g);
        let parts = [];

        const splitWordRe = new RegExp("(.|[\\r\\n]){1," + partLength + "}", "g");

        function splitLongWord(word) {
            return word.match(splitWordRe);
        }

        let j = 0;
        parts[0] = "";

        let partsLengthSum = 0;
        for (let i = 0; i < words.length; i++) {
            if (parts[j].length + words[i].length <= partLength) {
                parts[j] += words[i];
                continue;
            }

            partsLengthSum += parts[j].length;

            if (words[i].length <= partLength) {
                j++;
                parts[j] = words[i];
            } else {
                const subwords = splitLongWord(words[i]);
                j += subwords.length;
                parts = parts.concat(subwords);
            }
        }
        return parts;
    }
);

export {PostBodyView}
