// file: PostReferenceView.js
import {ChanPostView} from './ChanPostView.js'
import * as urlService from "../services/url_service";
import * as repo from "../repository";
import * as settings from "../services/settings_service";
import {highlightCode} from "../extra_services/hljs_service";
import {toShortcutUuid} from "../services/uuid_service";
import {showNotification} from "./notifications/notification_v";
import {config} from "../config";

const PostReferenceView = function () {
    const that = this;
    let span;
    const requestTimeout = 50; // ms
    const timeoutBeforeRemoving = 20;// ms

    that.create = (postId) => {
        span = document.createElement("span");
        span.className = "msgref";

        setPostId(postId);
        const link = document.createElement("a");
        link.href = urlService.getPostLinkUrl(postId);
        link.appendText(`@${toShortcutUuid(postId)}`);
        span.appendChild(link);
    };

    that.initialize = (element) => span = element;

    that.setMouseEvents = () => {
        function getQuotePreview(postId) {
            return document.querySelector(
                `.quote-preview[data-post-id='${postId}']`
            );
        }

        let timeoutId;

        span.onmouseenter = (e) => {
            const postId = getPostId();

            if (getQuotePreview(postId)) {
                return;
            }
            if (e.relatedTarget
                && e.relatedTarget.classList.contains("quote-preview")
            ) {
                return;
            }
            setWaitCursor();
            // console.log(e.target)

            const clientRect = span.getBoundingClientRect();

            const left = //(hasPageScrollbar()
                //?
                span.offsetLeft + span.offsetParent.offsetLeft
                //: clientRect.left)
                + clientRect.width / 2;

            const top = (hasPageScrollbar()
                    ? span.offsetTop + span.offsetParent.offsetTop
                    : clientRect.top
            ) + clientRect.height / 2;

            timeoutId = setTimeout(() => {
                showQuotePreview(postId, clientRect, left, top)
                    .then(resetWaitCursor, resetWaitCursor);
            }, requestTimeout);
        };

        span.onmouseleave = function (e) {
            clearTimeout(timeoutId);
            // if (e.relatedTarget.classList.contains(chan.classNames.quotePreview)) {
            //     return;
            // }
            const postId = getPostId();

            const quotePreview = getQuotePreview(postId);
            const relatedQuotePreview = e.relatedTarget.getClosestByClassName("quote-preview");
            if (quotePreview && relatedQuotePreview !== quotePreview) {
                // console.log("remove", e, e.relatedTarget)
                setTimeout(() => {
                    quotePreview.remove();
                }, 10);
            }
            // chan.removeQuotePreviews();
        };

        removeQuotePreviewsOnClick();

        span.onclick = (e) => {
            const postId = getPostId();
            const hash = `#p${postId}`;
            if (!document.querySelector(hash)) return;
            e.preventDefault();

            location.href = location.pathname + location.search + hash;
        }
    };

    function showQuotePreview(postId, referenceSpanClientRect, left, top) {
        const postPromise = repo.getPost(postId);

        return postPromise.then(post => {
            const postView = new ChanPostView(post, false, true);

            const quotePreview = document.createElement("div");
            quotePreview.className = "quote-preview";
            quotePreview.setAttribute("data-post-id", postId);
            postView.appendTo(quotePreview);

            // find out the horizontal position
            if (left < document.body.clientWidth / 2) {
                // quotePreview will be on the right
                quotePreview.style.left = left + "px";
                // console.log(Math.floor(document.body.clientWidth - left - 10));
                quotePreview.style.width = Math.floor(document.body.clientWidth - left - 10) + "px";
            } else {
                // on the left
                quotePreview.style.right = (document.body.clientWidth - left) + "px";
                quotePreview.style.width = Math.floor(left - 10) + "px";
            }

            // find out the vertical position
            quotePreview.style.top = 0; // prevent appearing of the scrollbar (on the chat page).

            // const ctr = document.getElementById("container");
            // ctr.style.width = getComputedStyle(ctr, null).width;

            document.body.appendChild(quotePreview); // quotePreview.clientHeight is unknown for a hidden element.
            if (referenceSpanClientRect.top < window.innerHeight / 2) {
                // below
                const quotePreviewHeight = getQuotePreviewHeight(quotePreview);
                if (quotePreviewHeight > window.innerHeight - top) {
                    const postBody = quotePreview
                        .getElementsByClassName("postbody")[0];

                    const height = Math.floor(
                        postBody.offsetHeight - quotePreviewHeight + window.innerHeight - top
                    );

                    postBody.style.height = `${height}px`;
                }
                quotePreview.style.top = `${top}px`;
                // console.log(quotePreviewHeight, window.innerHeight - top);
                // console.log("top=",referenceSpanClientRect)
            } else {
                // above
                const quotePreviewHeight = getQuotePreviewHeight(quotePreview);
                if (quotePreviewHeight > top) {
                    const postBody = quotePreview
                        .getElementsByClassName("postbody")[0];

                    const height = Math.floor(
                        postBody.offsetHeight - quotePreviewHeight + top
                    );

                    postBody.style.height = `${height}px`;
                }
                quotePreview.style.top = (top + 2 - quotePreview.clientHeight - referenceSpanClientRect.height) + "px";
                // console.log(referenceSpanClientRect.top);
            }
            // document.body.appendChild(quotePreview);
            quotePreview.onmouseleave = (e) => {
                if (!e.relatedTarget) return;
                const relatedQuotePreview = e.relatedTarget.getClosestByClassName("quote-preview");
                const quotePreviews = document.getElementsByClassName("quote-preview");

                const indexOfThisQuotePreview = quotePreviews.indexOf(e.target);
                const indexOfRelatedQuotePreview = quotePreviews.indexOf(relatedQuotePreview);

                if (indexOfRelatedQuotePreview > indexOfThisQuotePreview
                    || e.relatedTarget.classList.contains("msgref")) {
                    return;
                }
                // console.log("asdf", e.relatedTarget, indexOfRelatedQuotePreview, indexOfThisQuotePreview);

                setTimeout(() => e.target.remove(), timeoutBeforeRemoving);

                // const ctr = document.getElementById("container");
                // ctr.style.width = "";
            };

            if (settings.doHighlightCode()) {
                highlightCode(quotePreview);
            }
        }).catch(errorResponse => {
            showNotification(config.notification.type.error, errorResponse.message);
        });
    }

    const existingOnClickEventHandler = document.body.onclick || (x => x);

    function removeQuotePreviewsOnClick() {
        document.body.onclick = (e) => {
            existingOnClickEventHandler(e);

            if (e.target.getClosestByClassName("quote-preview")
                || e.target.classList.contains("msgref")) {
                return;
            }
            removeQuotePreviews();
        }
    }

    function removeQuotePreviews(e) {
        // http://stackoverflow.com/a/23989142
        const quotePreviews = document.querySelectorAll(".quote-preview");

        quotePreviews.forEach(quotePreview => quotePreview.remove());

        // const ctr = document.getElementById("container");
        // ctr.style.width = "";
    }

    function getPostId() {
        return span.getAttribute("data-post-id");
    }

    function setPostId(postId) {
        span.setAttribute("data-post-id", postId);
    }

    that.getOuterHtml = () => {
        return span.outerHTML;
    };

    function hasPageScrollbar() {
        return document.body.offsetHeight > innerHeight;
    }

    function getQuotePreviewHeight(quotePreview) {
        const quotePreviewStyle = getComputedStyle(quotePreview);
        const quotePreviewHeight = quotePreview.offsetHeight
            + parseFloat(quotePreviewStyle.marginTop.slice(0, -2));
        // + parseFloat(quotePreviewStyle.paddingTop.slice(0, -2))
        // + parseFloat(quotePreviewStyle.marginTop.slice(0, -2))
        // + parseFloat(quotePreviewStyle.paddingBottom.slice(0, -2));
        return quotePreviewHeight;
    }

    function setWaitCursor() {
        const link = span.firstChild;
        link.style.cursor = "wait";
    }

    function resetWaitCursor() {
        const link = span.firstChild;
        link.style.cursor = "";
    }
};

export {PostReferenceView}
