// file: repository.js
import * as httpService from "./services/http_service";
import * as pcService from "./services/post_cache_service";
import {AppEventSource} from "./app_event_source";
import {getSubscribeApiUrl} from "./services/url_service";
import {getOrCreateClientId} from "./services/client_service";

export function getRecentPosts() {
    return httpService.getRecentPosts().then(r => r.data).then(pcService.saveMany);
}

export function getPostsBefore(postId) {
    return httpService.getPostsBefore(postId).then(r => r.data).then(pcService.saveMany);
}

export function getPostsAfter(postId) {
    return httpService.getPostsAfter(postId).then(r => r.data).then(pcService.saveMany);
}

export function getLogsPage(pageNumber) {
    return httpService.getLogsPage(pageNumber).then(r => {
        pcService.saveMany(r.data);
        return r;
    });
}

export function getPost(postId, includeIndex = false) {
    const cachedPost = pcService.get(postId);
    if (cachedPost && !includeIndex) return Promise.resolve(cachedPost);

    return httpService.getPost(postId, includeIndex).then(r => r.data).then(pcService.save);
}

export function getThreadPage(threadId, pageNumber = null) {
    // null means the last page

    return httpService.getThreadPage(threadId, pageNumber).then(r => {
        r.data.op.thread = r.data.thread;

        r.data.posts = r.data.posts.map(p => {
            p.thread = r.data.thread;
            return p;
        });
        pcService.saveForThreadSlice(r.data);
        return r;
    });
}

export function getWarpPage(boardName = null, pageNumber = 0) {
    return httpService.getWarpPage(boardName, pageNumber).then(r => {
        r.data.shellThreads = r.data.shellThreads.map(st => {
            st.thread.board = st.thread.board || r.data.board;
            st.op.thread = st.thread;

            st.posts = st.posts.map(p => {
                p.thread = st.thread;
                return p;
            });
            return st;
        });
        pcService.saveForWarpSlice(r.data);
        return r;
    });
}

export function getServerInfo() {
    return httpService.getServerInfo().then(r => r.data);
}

export function createUser(newUser) {
    return httpService.createUser(newUser).then(r => r.data);
}

export function updateBoardDescription(board) {
    return httpService.updateBoardDescription(board).then(r => r.data);
}

export function updateHiddenByOp(updatedHiddenByOp) {
    return getOrCreateClientId().then(clientId => {
        updatedHiddenByOp.clientId = clientId;

        return httpService.updateHiddenByOp(updatedHiddenByOp).then(r => r.data);
    });
}

export function publish(newPost) {
    return httpService.publish(newPost).then(r => r.data);
}

export function authorize(auth) {
    return httpService.authorize(auth).then(r => r.data)
}


let eventSource;

export function subscribe() {
    eventSource = new AppEventSource(getSubscribeApiUrl());
}

export function addErrorEventListener(listener) {
    return eventSource.addErrorEventListener(listener);
}

export function addPostsListener(listener) {
    return eventSource.addPostsListener((e, post) => {
        // ignore ephemeral posts
        if (!post.ephemeral) return;

        listener(e, pcService.save(post));
    });
}

export function addBoardDescriptionListener(listener) {
    return eventSource.addBoardDescriptionListener(listener);
}

export function addConnectedEventListener(listener) {
    return eventSource.addConnectedEventListener(listener);
}

export function continueSubscription(subscriptionId) {
    return httpService.continueSubscription(subscriptionId);
}
