// file: http_service.js
import {
    parseAuthInfo,
    parseErrorResponse,
    parsePost, parseServerInfo, parseSuccessResponse, parseThreadSlice, parseWarpSlice,
} from "../parsers";
import {
    getAuthorizeApiUrl,
    getContinueSubscriptionApiUrl, getCreateUserApiUrl, getLogsPageApiUrl, getPostApiUrl, getPostsAfterApiUrl,
    getPostsBeforeApiUrl, getPublishApiUrl, getRecentPostsApiUrl, getServerInfoApiUrl,
    getThreadPageApiUrl, getUpdateBoardDescriptionApiUrl, getUpdateHiddenByOpApiUrl, getWarpPageApiUrl,
} from "./url_service";
import {RequestBody} from "../models";
import {getAccessToken} from "./access_token_service";

export function getRecentPosts() {
    return getJson(getRecentPostsApiUrl()).thenA(ps => ps.map(parsePost).reverse());
}

export function getPostsBefore(postId) {
    return getJson(getPostsBeforeApiUrl(postId)).thenA(ps => ps.map(parsePost).reverse());
}

export function getPostsAfter(postId) {
    return getJson(getPostsAfterApiUrl(postId)).thenA(ps => ps.map(parsePost));
}

export function getLogsPage(pageNumber) {
    return getJson(getLogsPageApiUrl(pageNumber)).thenA(ps => ps.map(parsePost));
}

export function getPost(postId, includeIndex) {
    return getJson(getPostApiUrl(postId, includeIndex)).thenA(parsePost);
}

export function getThreadPage(threadId, pageNumber) {
    // null means the last page

    return getJson(getThreadPageApiUrl(threadId, pageNumber)).thenA(parseThreadSlice);
}

export function getWarpPage(boardName, pageNumber) {
    return getJson(getWarpPageApiUrl(boardName, pageNumber)).thenA(parseWarpSlice);
}

export function getServerInfo() {
    return getJson(getServerInfoApiUrl()).thenA(parseServerInfo);
}

export function updateBoardDescription(board) {
    return patchJson(getUpdateBoardDescriptionApiUrl(), new RequestBody(board)).thenA(message => message);
}

export function updateHiddenByOp(updatedHiddenByOp) {
    return patchJson(getUpdateHiddenByOpApiUrl(), new RequestBody(updatedHiddenByOp)).thenA(message => message);
}

export function publish(newPost) {
    return postJson(getPublishApiUrl(), new RequestBody(newPost)).thenA(p => parsePost(p));
}

export function authorize(auth) {
    return postJson(getAuthorizeApiUrl(), new RequestBody(auth)).thenA(parseAuthInfo);
}

export function createUser(newUser) {
    return postJson(getCreateUserApiUrl(), new RequestBody(newUser)).thenA(message => message);
}

export function continueSubscription(subscriptionId) {
    return postJson(getContinueSubscriptionApiUrl(subscriptionId)).thenA(message => message);
}


function getJson(url) {
    return handleFetchPromise(fetch(url));
}

function postJson(url, data) {
    return httpRequestJson(url, 'POST', data);
}

function patchJson(url, data) {
    return httpRequestJson(url, 'PATCH', data);
}


function httpRequestJson(url, method, data) {
    let headers = {
        'Accept': 'application/json',
        'Content-Type': 'application/json',
    };
    if (['POST', 'PATCH', 'DELETE'].includes(method)) {
        const accessToken = getAccessToken();
        if (accessToken) headers['Authorization'] = accessToken;
    }
    return handleFetchPromise(fetch(url, {
        method: method,
        headers,
        body: JSON.stringify(data)
    }));
}

function handleFetchPromise(fetchPromise) {
    return fetchPromise.then(response =>
        Promise.all([Promise.resolve(response), response.json()])
    ).then(([response, json]) => {
        if (response.ok) return json;

        throw parseErrorResponse(json);
    }).then(
        json => parseSuccessResponse(json)
    );
}