// file: local_storage_service.js
import * as storageService from "./storage_service";
import * as customEvents from "../../custom_events";

// because of this bug https://bugzilla.mozilla.org/show_bug.cgi?id=1453699
// which affects Tor Browser users
// we need a temporary workaround: we ask other open tabs to send their localStorage to this tab
// if we don't receive anything during 20ms we assume that there're no other open tabs.
// we use fake localStorage object and synchronize it across tabs using BroadcastChannel.
// When this bug is fixed in Tor Browser, this code will be removed.
const isLsBugWorkaroundNeeded = (
        navigator.userAgent ===
        "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:68.0) Gecko/20100101 Firefox/68.0"
        || navigator.userAgent ===
        "Mozilla/5.0 (Windows NT 10.0; rv:68.0) Gecko/20100101 Firefox/68.0"
        || navigator.userAgent ===
        "Mozilla/5.0 (Windows NT 6.1; rv:60.0) Gecko/20100101 Firefox/60.0"
        || navigator.userAgent ===
        "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:78.0) Gecko/20100101 Firefox/78.0"
);
const wa = {}; // workaround

export function initLocalStorageService(callback) {
    const onInitialized = () => {
        addStorageEventListener();
        callback();
    };

    if (isLsBugWorkaroundNeeded) {
        wa.fakeLocalStorage = {};
        Object.keys(localStorage).forEach(key => wa.fakeLocalStorage[key] = localStorage[key]);

        wa.bc = new BroadcastChannel('ls');
        wa.bc.postMessage({type: 'send your ls'});

        let isLsReceived = false;
        wa.bc.addEventListener('message', e => {
            if (e.data.type === 'send your ls') {
                wa.bc.postMessage({type: 'ls', localStorage: wa.fakeLocalStorage});
            }

            if (e.data.type === 'ls') {
                if (isLsReceived) return;

                isLsReceived = true;
                const receivedLs = e.data.localStorage;
                Object.keys(receivedLs).forEach(key => {
                    if (receivedLs[key] !== undefined) {
                        wa.fakeLocalStorage[key] = receivedLs[key];
                        localStorage[key] = receivedLs[key];
                    }
                });
                onInitialized();
            }

            // storage event emulation
            if (e.data.type === 'storage') {
                if (e.data.newValue === undefined) {
                    delete wa.fakeLocalStorage[e.data.key];
                    delete localStorage[e.data.key];
                } else {
                    wa.fakeLocalStorage[e.data.key] = e.data.newValue;
                    localStorage[e.data.key] = e.data.newValue;
                }
                onStorageHandler(e.data);
            }
        });

        setTimeout(() => {
            if (!isLsReceived) {
                console.log('timeout');
                isLsReceived = true;
                onInitialized();
            }
        }, 20);
    } else {
        onInitialized();
    }
}

export function getItem(key, defaultValue) {
    if (isLsBugWorkaroundNeeded) {
        return storageService.parseValue(wa.fakeLocalStorage[key], defaultValue);
    }

    return storageService.getItem(localStorage, key, defaultValue);
}

export function getItems(keyRe) {
    if (isLsBugWorkaroundNeeded) {
        return Object.keys(wa.fakeLocalStorage).filter(key => keyRe.test(key)).map(key => {
            return {key, value: getItem(key)};
        });
    }

    return storageService.getItems(localStorage, keyRe);
}

export function setItem(key, value, defaultValue) {
    const oldValue = getItem(key, defaultValue);

    if (isLsBugWorkaroundNeeded) {
        const oldValueStr = storageService.serializeValue(oldValue);
        const newValueStr = storageService.serializeValue(value);
        wa.fakeLocalStorage[key] = newValueStr;

        if (oldValueStr !== newValueStr) {
            // console.log('send newVal', newValueStr);
            wa.bc.postMessage({
                type: 'storage', key,
                oldValue: oldValueStr,
                newValue: newValueStr,
            });
        }
    }

    storageService.setItem(localStorage, key, value);

    // todo: this won't work for objects
    if (oldValue !== value) customEvents.lsChange(key, oldValue, value);
}

export function toggle(key, defaultValue = false) {
    const oldValue = getItem(key, defaultValue);

    if (isLsBugWorkaroundNeeded) {
        const oldValueStr = storageService.serializeValue(oldValue);
        const newValueStr = storageService.serializeValue(!getItem(key, false));
        wa.fakeLocalStorage[key] = newValueStr;

        wa.bc.postMessage({
            type: 'storage', key,
            oldValue: oldValueStr,
            newValue: newValueStr,
        });
    }

    storageService.toggle(localStorage, key);

    customEvents.lsChange(key, oldValue, !oldValue);
}

export function removeItem(key) {
    if (isLsBugWorkaroundNeeded) {
        const oldValueStr = wa.fakeLocalStorage[key];
        delete wa.fakeLocalStorage[oldValueStr];

        wa.bc.postMessage({
            type: 'storage', key,
            oldValue: oldValueStr,
        });
    }

    storageService.removeItem(localStorage, key);
}

function addStorageEventListener() {
    if (isLsBugWorkaroundNeeded) return;

    window.addEventListener('storage', onStorageHandler, false);
}

function onStorageHandler(e) {
    // console.log('received newVal', e.newValue);
    // console.log('onStorage', e.key, e.oldValue, e.newValue); // todo: delete
    const oldValue = storageService.parseValue(e.oldValue, null);
    const newValue = storageService.parseValue(e.newValue, null);

    customEvents.lsChange(e.key, oldValue, newValue, true);
}
